// CHydro.h: interface for the Hydro class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HYDRO_H__66CB5183_5011_11D8_9D25_005056063E62__INCLUDED_)
#define AFX_HYDRO_H__66CB5183_5011_11D8_9D25_005056063E62__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "constants.h"
#include "tnt.h"
#include <comdef.h>
#include "HydroConstants.h"

//#include "CEmulsion.h"
//#include <CHydroXML.h>
//#include "CHydroData.h"


#pragma  warning (disable: 4290)

enum EHydroFlowPatterns {
 UNDEFINED  = 7777777,
 MONO_GAS   = 10010000,
 MONO_OIL   = 10020000,
 MONO_WAT   = 10030000,
 STRA_SMO   = 20010000,
 STRA_WAV   = 20020000,
 ANNULAR    = 20030000,
 DISPERSE   = 20110000,
 ANN_DISP   = 20120000,
 SLUGFLOW   = 20210000,
 LIQ_LIQ    = 25000000,
 O_DIS_STSM = 31110000,
 O_DIS_STWA = 31120000,
 O_DIS_AN   = 31130000,
 O_DIS_DISP = 31210000,
 O_DIS_SLUG = 31310000,
 W_DIS_STSM = 32110000,
 W_DIS_STWA = 32120000,
 W_DIS_AN   = 32130000,
 W_DIS_DISP = 32210000,
 W_DIS_SLUG = 32310000,
 ST_STSM    = 33110000,
 ST_STWA    = 33120000,
 ST_AN      = 33130000,
 ST_ANWA    = 33140000,
 ST_ANDI    = 33150000,
 STSM_STSM  = 33210000,
 STSM_STWA  = 33220000,
 STWA_STSM  = 33310000,
 STWA_STWA  = 33320000,
 STMI_STSM  = 33410000,
 STMI_STWA  = 33420000,
 ST_SLUG    = 33510000
};

enum ECheckBase
{
	Uk,
	RkVm,
	RkUm
};

/** @brief Hydro Interface

	@author M.Gainville - TINA Project - Institut Franais du Ptrole - 2005 */
// ======================================================================================
//class __declspec(dllexport) CHydro
class CHydro{
public:
	
	/** Constructors & Destructor */
	//@{
	//static CHydro* loadModel(CHydroData* pHydroData);
	/** @brief Constructor of the Virtual Class.*/
	CHydro();
	//CHydro(CHydroData* phydroData);

	/** Destructor */
	virtual ~CHydro();
	//@}


	
	/// <summary>Pure virtual method to define which hydro is in use</summary>
	/// <returns>COMP or SYNT or SIMP or OLGA</returns>
	/// <exception>No exception</exception>
	virtual enum hydroCst::modelHydro get_hydroID() throw() = 0;
	
	/** Methods */
	//@{
	/** Get absolute velocities*/
	TNT::Array1D<double> const* getAbsoluteVelocities() const;
	/** Get superficial velocities*/
	TNT::Array1D<double> const* getSuperficialVelocities() const;
	/** Get local volumetric fractions*/
	TNT::Array1D<double> const* getLocalVolumetricFractions() const;
	/** Get barycentric velocity*/
	double getBarycentricVelocity() const;
	/** Get local pressure drop (friction losses and gravities forcies)*/
	double getLocalPressureDrop() const;
	/** Get friction pressure drop*/
	double getFrictionPressureDrop() const;
	/** Get gravity pressure drop*/
	double getGravityPressureDrop() const;
	/** Get Internal Fluid Resistivity*/
	double getInternalFluidResistivity() const;
	/** Get flow pattern*/
	EHydroFlowPatterns getFlowPattern() const;
	/** Get slug fraction*/
	double getSlugFraction() const;
	/** Get effective liquid  viscosity*/
	TNT::Array1D<double> const* getEffectiveLiquidViscosity()  const;

	/** Get derivatives of absolute velocities versus local volumetric fractions*/
	TNT::Array2D<double> const* getAbsoluteVelocitiesDLocalVolumetricFractions() const;
	/** Get derivatives of absolute velocities versus phase densities*/
	TNT::Array2D<double> const* getAbsoluteVelocitiesDDensities() const;
	/** Get derivatives of absolute velocities versus phase viscosities*/
	TNT::Array2D<double> const* getAbsoluteVelocitiesDViscosities() const;
	/** Get derivatives of absolute velocities versus surface tensions*/
	TNT::Array3D<double> const* getAbsoluteVelocitiesDSurfaceTensions() const;
	/** Get derivatives of absolute velocities versus the barycentric velocity*/
	TNT::Array1D<double> const* getAbsoluteVelocitiesDBarycentricVelocity() const;
	/** Get derivatives of absolute velocities versus the mixture velocity */
	TNT::Array1D<double> const* getAbsoluteVelocitiesDMixtureVelocity() const;

	/** Get derivatives of the local pressure drop versus local volumetric fractions*/
	TNT::Array1D<double> const* getLocalPressureDropDLocalVolumetricFractions() const;
	/** Get derivatives of the local pressure drop versus phase densities*/
	TNT::Array1D<double> const* getLocalPressureDropDDensities() const;
	/** Get derivatives of the local pressure drop versus phase viscosities*/
	TNT::Array1D<double> const* getLocalPressureDropDViscosities() const;
	/** Get derivatives of the local pressure drop versus surface tensions*/
	TNT::Array2D<double> const* getLocalPressureDropDSurfaceTensions() const;
	/** Get derivative of the local pressure drop versus the barycentric velocity*/
	double getLocalPressureDropDBarycentricVelocity() const;
	/** Get derivative of the local pressure drop versus the mixture velocity */
	double getLocalPressureDropDMixtureVelocity() const;

	/** Get derivatives of the internal fluid resistivity versus local volumetric fractions*/
	TNT::Array1D<double> const* getInternalFluidResistivityDLocalVolumetricFractions() const;
	/** Get derivatives of the internal fluid resistivity versus phase densities*/
	TNT::Array1D<double> const* getInternalFluidResistivityDDensities() const;
	/** Get derivatives of the internal fluid resistivity versus phase viscosities*/
	TNT::Array1D<double> const* getInternalFluidResistivityDViscosities() const;
	/** Get derivatives of the internal fluid resistivity versus surface tensions*/
	TNT::Array2D<double> const* getInternalFluidResistivityDSurfaceTensions() const;
	/** Get derivatives of the internal fluid resistivity versus phase heat capacities*/
	TNT::Array1D<double> const* getInternalFluidResistivityDHeatCapacities() const;
	/** Get derivatives of the internal fluid resistivity versus phase conductivities*/
	TNT::Array1D<double> const* getInternalFluidResistivityDConductivities() const;
	/** Get derivative of the internal fluid resistivity versus the barycentric velocity*/
	double getInternalFluidResistivityDBarycentricVelocity() const;

	/** @brief Useful function to initialize Hydro Model before computing.
		By default, do nothing. */
	virtual void initialize() {};


	/** @brief Performe hydrodynamic calculations knowing the sperficial velocities.
	@param phaseQualifiers : the phase name identifier of the present phases, dimension equal to nbrOfPresentPhases
	@param roughness : the relative roughness of the pipe section
	@return . */
	virtual void hydroUk(double diameter, double roughness, double slopeSinus, int nbrOfPresentPhases,
						const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &superficialVelocities,
						const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
						const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
						const TNT::Array1D<double> &conductivities) throw (...) =0;
	/** @brief Performe hydrodynamic calculations knowing the phase volumetric fractions and the barycentric velocities.
	@param phaseQualifiers : the phase ID of the present phases, dimension equal to nbrOfPresentPhases
	@param roughness : the relative roughness of the pipe section
	@return . */
	virtual HRESULT hydroRkVm(const double diameter,const double roughness,const double slopeSinus,const int nbrOfPresentPhases,
						const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &localVolumetricFractions,
						double barycentricVelocity,
						const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
						const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
						const TNT::Array1D<double> &conductivities) throw (...) =0;
	/** @brief Perform hydrodynamic calculations knowing the phase volumetric fractions & the mixture Velocity.
		@param diameter   Pipe diameter
		@param roughness  Pipe roughness
		@param slopeSinus Pipe slope sinus
		@param nbrOfPresentPhases
		@param 
		@return */
	virtual HRESULT hydroRkUm(const double diameter,
							  const double roughness,
							  const double slopeSinus,
							  const int    nbrOfPresentPhases,
							  const TNT::Array1D<EPhases> &phaseQualifiers,
							  const TNT::Array1D<double>  &volumetricFractions,
							  const double            mixtureVelocity,
							  const TNT::Array1D<double>  &densities,
							  const TNT::Array1D<double>  &viscosities,
							  const TNT::Array2D<double>  &surfaceTensions,
							  const TNT::Array1D<double>  &heatCapacities,
							  const TNT::Array1D<double>  &conductivities) throw (...) = 0;

	/** @brief Compute hydrodynamic properties for a given set of properties.
	@return . */
	virtual HRESULT computeProperties() throw (...) =0;	
	
	/** @brief Compute hydrodynamic derivatives.
	@return . */
	virtual HRESULT computeDerivatives() throw (...) =0;

	//@}
	//static CHydro* createHydro(enum hydroCst::modelHydro whichHydro) throw();


protected:

	/// <summary>initialize hydro model given a data model</summary>
	/// <param name="phydroData">the data model</param>
	/// <returns>void</returns>
	/// <remarks>method is called from loadModel method</remarks>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	//virtual void initialize(CHydroData* phydroData) throw(...) = 0;


	
	/** Methods */
	//@{
	/** @brief Initialise hydrodynamic output data before hydodynamic calculations.
	@return . */
	void initialiseHydroOutputs();
	//@}

	
	
	/** Parameters */
	//@{
	/** Hydro Function base */
	ECheckBase m_whichBase;

	/** diameter */
	double				m_diameter;
	/** sinus of the slope*/
	double				m_slopeSinus;
	/** internal relative roughness*/
	double				m_roughness;
	/** number of present phases*/
	int					m_nbrOfPresentPhases;
	/** Phase Qualifiers of all phases in the mixture. */
	TNT::Array1D<EPhases>		m_phaseQualifiers;
	/** phase qualifers */
	//TNT::Array1D<int>		m_phaseQualifiers;
	/** local phase volumetric fractions*/
	TNT::Array1D<double>		m_localVolumetricFractions;
	/** flowing phase volumetric fractions*/
	TNT::Array1D<double>		m_flowingVolumetricFractions;
	/** phase densities*/
	TNT::Array1D<double>		m_densities;
	/** phase dynamic viscosities*/
	TNT::Array1D<double>		m_viscosities;
	/** phase surface tensions*/
	TNT::Array2D<double>		m_surfaceTensions;
	/** phase heat capacities*/
	TNT::Array1D<double>		m_heatCapacities;
	/** phase conductivities*/
	TNT::Array1D<double>		m_conductivities;
	/** phase superficial velocities*/
	TNT::Array1D<double>		m_superficialVelocities;
	/** phase absolute velocities*/
	TNT::Array1D<double>		m_absoluteVelocities;
	/** phase barycentric velocity*/
	double				m_barycentricVelocity;
	/** phase mixture velocity*/
	double				m_mixtureVelocity;
	/** flow pattern*/
	EHydroFlowPatterns	m_flowPattern;
	/** local pressure drop*/
	double				m_localPressureDrop;
	/** local pressure drop corresponding to friction losses*/
	double				m_frictionPressureDrop;
	/** local pressure drop corresponding to gravity losses*/
	double				m_gravityPressureDrop;
	/** internal fluid resistivity = 1/PI/lambda/Nu = 1/PI/D/hFluid*/
	double				m_internalFluidResistivity;
	/** slug fraction*/
	double				m_slugFraction;
	/** effective liquid viscosity */
	TNT::Array1D<double> m_effectiveLiquidViscosity;
	//@}


	/** Derivatives */
	//@{
	/** derivatives of absolute velocities versus local volumetric fractions*/
	TNT::Array2D<double>		m_absoluteVelocitiesDLocalVolumetricFractions;
	/** derivatives of absolute velocities versus phase densities*/
	TNT::Array2D<double>		m_absoluteVelocitiesDDensities;
	/** derivatives of absolute velocities versus phase viscosities*/
	TNT::Array2D<double>		m_absoluteVelocitiesDViscosities;
	/** derivatives of absolute velocities versus surface tensions*/
	TNT::Array3D<double>		m_absoluteVelocitiesDSurfaceTensions;
	/** derivatives of absolute velocities versus the barycentric velocity*/
	TNT::Array1D<double>		m_absoluteVelocitiesDBarycentricVelocity;
	/** derivatives of absolute velocities versus the mixture velocity */
	TNT::Array1D<double>		m_absoluteVelocitiesDMixtureVelocity;
	/** derivatives of the local pressure drop versus local volumetric fractions*/
	TNT::Array1D<double>		m_localPressureDropDLocalVolumetricFractions;
	/** derivatives of the local pressure drop versus phase densities*/
	TNT::Array1D<double>		m_localPressureDropDDensities;
	/** derivatives of the local pressure drop versus phase viscosities*/
	TNT::Array1D<double>		m_localPressureDropDViscosities;
	/** derivatives of the local pressure drop versus surface tensions*/
	TNT::Array2D<double>		m_localPressureDropDSurfaceTensions;
	/** derivative of the local pressure drop versus the barycentric velocity*/
	double				m_localPressureDropDBarycentricVelocity;
	/** derivative of the local pressure drop versus the mixture velocity */
	double				m_localPressureDropDMixtureVelocity;
	/** derivatives of the internal fluid resistivity versus local volumetric fractions*/
	TNT::Array1D<double>		m_internalFluidResistivityDLocalVolumetricFractions;
	/** derivatives of the internal fluid resistivity versus phase densities*/
	TNT::Array1D<double>		m_internalFluidResistivityDDensities;
	/** derivatives of the internal fluid resistivity versus phase viscosities*/
	TNT::Array1D<double>		m_internalFluidResistivityDViscosities;
	/** derivatives of the internal fluid resistivity versus surface tensions*/
	TNT::Array2D<double>		m_internalFluidResistivityDSurfaceTensions;
	/** derivatives of the internal fluid resistivity versus phase heat capacities*/
	TNT::Array1D<double>		m_internalFluidResistivityDHeatCapacities;
	/** derivatives of the internal fluid resistivity versus phase conductivities*/
	TNT::Array1D<double>		m_internalFluidResistivityDConductivities;
	/** derivative of the internal fluid resistivity versus the barycentric velocity*/
	double				m_internalFluidResistivityDBarycentricVelocity;

	//@}
	
};

#endif // !defined(AFX_HYDRO_H__66CB5183_5011_11D8_9D25_005056063E62__INCLUDED_)
