#include "CHydroData.h"
#include "ExceptionUtils.h"
#include "constants.h"

/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
CHydroData::CHydroData(enum hydroCst::modelHydro modelHydro, enum hydroCst::modelEmulsion modelEmulsion, double criticalWC, double ronningsenTemp, double palAndRhodesCst, const TNT::Array1D<CExperimentalPointData*>& expPoints, double tau0, double x): m_expPoints(expPoints.dim()){
	m_modelHydro = modelHydro;
	m_modelEmulsion = modelEmulsion;
	m_criticalWC = criticalWC;
	m_ronningsenTemp = ronningsenTemp;
	m_palAndRhodesCst = palAndRhodesCst;
	m_tau0 = tau0;
	m_x = x;
	for(int i = 0; i < expPoints.dim(); ++i)
		m_expPoints[i] = expPoints[i];
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
CHydroData::~CHydroData(){
	for(unsigned int i = 0; i < m_expPoints.size(); ++i)
		delete m_expPoints[i];
	m_expPoints.clear();
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
CHydroData::CHydroData(const CHydroData& hydro){
	m_modelHydro = hydro.m_modelHydro;
	m_modelEmulsion = hydro.m_modelEmulsion;
	m_criticalWC = hydro.m_criticalWC;
	m_ronningsenTemp = hydro.m_ronningsenTemp;
	m_palAndRhodesCst = hydro.m_palAndRhodesCst;
	size_t nbPoints = hydro.m_expPoints.size();
	m_expPoints.resize(nbPoints);
	m_tau0 = hydro.m_tau0;
	m_x = hydro.m_x;
	for(unsigned int i = 0; i < nbPoints; ++i)
		m_expPoints[i] = new CExperimentalPointData(*hydro.m_expPoints[i]);
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
enum hydroCst::modelHydro CHydroData::get_hydroModel() const{
	return m_modelHydro;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
enum hydroCst::modelEmulsion CHydroData::get_emulsionModel() const{
	return m_modelEmulsion;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
double CHydroData::get_criticalWC() const{
	return m_criticalWC;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
double CHydroData::get_ronningsenTemp() const{
	return m_ronningsenTemp;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
double CHydroData::get_palAndRhodesCst() const{
	return m_palAndRhodesCst;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
unsigned int CHydroData::get_nbExpPoints() const{
	return m_expPoints.size();
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
CExperimentalPointData* CHydroData::get_experimentPoint(unsigned int wantedPoint){
	unsigned int nbPoints = m_expPoints.size();
	R2PException::Utils::ThrowException(wantedPoint < nbPoints, R2PException::R2P_EINVARG, L"Invalid experimental point requested.", L"Fail to get element %d from experimental point list (size = %d).\n", wantedPoint, nbPoints);
	return m_expPoints[wantedPoint];
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_hydroModel(enum hydroCst::modelHydro aHydroModel){
	m_modelHydro = aHydroModel;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
double CHydroData::get_tau0() const{
	return m_tau0;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
double CHydroData::get_x() const{
	return m_x;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_tau0(double tau0){
	m_tau0 = tau0;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_x(double x){
	m_x = x;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_emulsionModel(enum hydroCst::modelEmulsion aModelEmulsion){
	m_modelEmulsion = aModelEmulsion;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_criticalWC(double wc){
	m_criticalWC = wc;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_ronningsenTemp(double ronningsenTemp){
	m_ronningsenTemp = ronningsenTemp;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CHydroData::set_palAndRhodesCst(double palNRhodesCst){
	m_palAndRhodesCst = palNRhodesCst;
}
