#include "CCOMUtils.h"

#include <objbase.h>
#include <ExceptionUtils.h>
#include <CRegisterInformation.h>


/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
int CCOMUtils::checkNRelease(IDispatch* pDisp){
	if(pDisp){
		return pDisp->Release();
	}
	return 0;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
HRESULT CCOMUtils::InitVariant4Copy(const VARIANT& fromVariant, VARIANT& toVariant, VARTYPE vt){
	HRESULT hr = S_OK;
	if(fromVariant.vt != vt)
		throw R2PException::CMyException(R2PException::R2P_EFAILEDINIT, L"Invalid Type.", L"Fail to convert type. Please contact support.\n");
	if(toVariant.vt != VT_EMPTY){
		//throw warning: may generate memory leaks
		hr = R2PException::R2P_SWARN;
	}
	toVariant.vt = vt;
	if(vt == VT_BSTR)
		toVariant.bstrVal = NULL;
	if((VT_ARRAY & vt) != 0)
		toVariant.parray = NULL;

	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CCOMUtils::loadCOM(){
	bool isCOMInit = true;
	HRESULT hr;
	try{
		std::wstring mode;
		CRegisterInformation::getInitialisationMode(mode);
	
		if(_wcsicmp(mode.c_str(), L"multithread") == 0){
			hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
			//if succeed: go to catch, otherwise try to initialize with default behavior.
			R2PException::Utils::ThrowException(FAILED(hr), R2PException::R2P_EFAILEDINIT, L"", L"");
		}
		if (_wcsicmp(mode.c_str(), L"speedovermemory") == 0){
			hr = CoInitializeEx(NULL, COINIT_SPEED_OVER_MEMORY);
			//if succeed: go to catch, otherwise try to initialize with default behavior.
			R2PException::Utils::ThrowException(FAILED(hr), R2PException::R2P_EFAILEDINIT, L"", L"");
		}
		hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
		R2PException::Utils::ThrowException(FAILED(hr), R2PException::R2P_EFAILEDINIT, L"", L"");
		isCOMInit = false;
	}catch(R2PException::CMyException&){
		//ok: victory
	}
	//Fail to init COM
	R2PException::Utils::ThrowException(isCOMInit == true, R2PException::R2P_EFAILEDINIT, L"Initialization error.", L"Fail to initialize the COM library.\n");
}
