// CDATAPACK.cpp: implementation of the CDATAPACK class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "CDATAPACK.h"
#include "CSafeArrayUtils.h"
#include <CMyException.h>
#include <ATLbase.h>

/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array1D<long>& arrayLng_in, VARIANT& varLng_out, bool isVariant){
	Array2Variant(arrayLng_in, varLng_out, (VARTYPE)(isVariant? VT_VARIANT: VT_I4));
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array1D<double>& arrayDbl_in, VARIANT& varDbl_out, bool isVariant){
	Array2Variant(arrayDbl_in, varDbl_out, (VARTYPE)(isVariant? VT_VARIANT: VT_R8));
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array1D<char*>& arrayChar_in, VARIANT& varChar_out, bool isVariant){
	Array2Variant(arrayChar_in, varChar_out, (VARTYPE)(isVariant? VT_VARIANT: VT_BSTR));
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array1D<std::wstring>& arrayWstring_in, VARIANT& varWstring_out, bool isVariant){
	Array2Variant(arrayWstring_in, varWstring_out, (VARTYPE)(isVariant? VT_VARIANT: VT_BSTR));
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::wstring2BSTR(const std::wstring& ws_in, BSTR& bstr_out){
	wchar2BSTR(ws_in.length() == 0 ? NULL : ws_in.c_str(), bstr_out);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::wchar2BSTR(const wchar_t* wchar_in, BSTR& bstr_out){

	if(wchar_in)
		bstr_out = ::SysAllocString(wchar_in);
	else
		bstr_out = NULL;//::SysAllocString(L"");
		
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array2D<long>& arrayLng_in, VARIANT& varLng_out, bool isVariant/*, long (*function)(long)*/){
	Array2Variant(arrayLng_in, varLng_out, (VARTYPE)(isVariant? VT_VARIANT: VT_I4)/*, &CCOMUtils::nop<long>*/);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array2D<double>& arrayDbl_in, VARIANT& varDbl_out, bool isVariant/*, void (*function)(const double*, double*&)*/){
	Array2Variant(arrayDbl_in, varDbl_out, (VARTYPE)(isVariant? VT_VARIANT: VT_R8)/*, function*/);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array2D<char*>& arrayChar_in, VARIANT& varChar_out, bool isVariant/*, char* (*function)(char*)*/){
	Array2Variant(arrayChar_in, varChar_out, (VARTYPE)(isVariant? VT_VARIANT: VT_BSTR)/*, &CCOMUtils::nop<char*>*/);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array2D<std::wstring>& arrayWstring_in, VARIANT& varWstring_out, bool isVariant/*, std::wstring (*function)(std::wstring)*/){
	Array2Variant(arrayWstring_in, varWstring_out, (VARTYPE)(isVariant? VT_VARIANT: VT_BSTR)/*, &CCOMUtils::nop<std::wstring>*/);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array2D<IDispatch*>& arrayDisp_in, VARIANT& varDisp_out, bool isVariant/*, IDispatch* (*function)(IDispatch*)*/){
	Array2Variant(arrayDisp_in, varDisp_out, (VARTYPE)(isVariant? VT_VARIANT: VT_DISPATCH)/*, &CCOMUtils::nop<IDispatch*>*/);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const TNT::Array1D<IDispatch*>& arrayDisp_in, VARIANT& varDisp_out, bool isVariant){
	Array2Variant(arrayDisp_in, varDisp_out, (VARTYPE)(isVariant? VT_VARIANT: VT_DISPATCH));
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Array2Variant(const wchar_t* array_in[], VARIANT& var_out, size_t len, bool isVariant){
	CopyArray1D2Variant(array_in, var_out, (VARTYPE)(isVariant? VT_VARIANT: VT_BSTR), (long)len);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& varDbl_in, TNT::Array1D<double>& arrayDbl_out){
	Variant2Array(varDbl_in, arrayDbl_out, VT_R8);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array1D<long>& array_out){
	Variant2Array(var_in, array_out, VT_I4);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array2D<long>& array_out){
	Variant2Array(var_in, array_out, VT_I4);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& varBSTR_in, TNT::Array1D<std::wstring>& arrayWStr_out){
	Variant2Array(varBSTR_in, arrayWStr_out, VT_BSTR);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array2D<std::wstring>& array_out){
	Variant2Array(var_in, array_out, VT_BSTR);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& varDbl_in, TNT::Array2D<double>& arrayDbl_out){
	Variant2Array(varDbl_in, arrayDbl_out, VT_R8);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& varBool_in, TNT::Array1D<bool>& arrayBool_out){
	Variant2Array(varBool_in, arrayBool_out, VT_BOOL);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& varBool_in, TNT::Array2D<bool>& arrayBool_out){
	Variant2Array(varBool_in, arrayBool_out, VT_BOOL);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array1D<IDispatch*>& array_out){
	Variant2Array(var_in, array_out, VT_DISPATCH);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::Variant2Array(const VARIANT& var_in, TNT::Array2D<IDispatch*>& array_out){
	Variant2Array(var_in, array_out, VT_DISPATCH);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::String2WString(const std::string& a_Src, std::wstring& a_Dest){
	String2WString(a_Src.c_str(), a_Dest);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::String2WString(const char* a_Src, std::wstring& a_Dest){
	if(a_Src){
		size_t len = strlen(a_Src);
		if(len != 0){
			wchar_t *pWChar = new wchar_t[len+1];
			mbstowcs_s(&len, pWChar, (size_t)(len+1), a_Src, (size_t)(len+1));
			a_Dest.assign(pWChar);
			delete [] pWChar;
		}
		else
			a_Dest = L"";
	}
	else
		a_Dest = L"";
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::WString2String(const std::wstring& a_Src, std::string& a_Dest){
	size_t len = a_Src.length();
	if(len != 0){
		char* ptmp = new char[len+1];
		wcstombs_s(&len, ptmp, len+1, a_Src.c_str(), len+1);
		a_Dest.assign(ptmp);
		delete [] ptmp;
	}
	else
		a_Dest = "";
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::BSTR2WString(const BSTR a_Src, std::wstring& a_Dest){
	a_Dest = a_Src == NULL? L"": a_Src;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
void CDATAPACK::wstring2double(const TNT::Array1D<std::wstring>& a_Src, TNT::Array1D<double>& a_Dest){
	int nbElem = a_Src.dim();
	a_Dest = TNT::Array1D<double>(nbElem, DOUBLE_INF);
	for(int i = 0; i < nbElem; ++i)
		a_Dest[i] = my_wtof(a_Src[i].c_str());
}
