// SimpleHydro.cpp: implementation of the CSimpleHydro class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CSimpleHydro.h"
#include <math.h>
#include <CMyException.h>
#include <ExceptionUtils.h>
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CSimpleHydro::~CSimpleHydro(){
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
CSimpleHydro::CSimpleHydro(): m_isPresentPhases(NBPHASES, false){
	m_frictionFactor = 0.005e0;
	unsigned int oilVal = (unsigned int)pow(2., OIL);
	unsigned int gasVal = (unsigned int)pow(2., GAS);
	unsigned int waterVal = (unsigned int)pow(2., WATER);
	//insert prop for GAS, OIL, WATER
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(oilVal + gasVal + waterVal, std::make_pair<EHydroFlowPatterns, double>(ST_SLUG, 0.5e0)));
	//insert prop for GAS, OIL
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(oilVal + gasVal, std::make_pair<EHydroFlowPatterns, double>(SLUGFLOW, 0.5e0)));
	//insert prop for GAS, WATER
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(gasVal + waterVal, std::make_pair<EHydroFlowPatterns, double>(SLUGFLOW, 0.5e0)));
	//insert prop for OIL, WATER
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(oilVal + waterVal, std::make_pair<EHydroFlowPatterns, double>(LIQ_LIQ, 0.)));
	//insert prop for GAS
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(gasVal, std::make_pair<EHydroFlowPatterns, double>(MONO_GAS, 1.)));
	//insert prop for OIL
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(oilVal, std::make_pair<EHydroFlowPatterns, double>(MONO_OIL, 0.)));
	//insert prop for WATER
	m_mapHydroProp.insert(std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::value_type(waterVal, std::make_pair<EHydroFlowPatterns, double>(MONO_WAT, 0.)));
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
void CSimpleHydro::hydroUk(double diameter, double roughness, double slopeSinus, int nbrOfPresentPhases,
						const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &superficialVelocities,
						const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
						const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
						const TNT::Array1D<double> &conductivities){

	// Initialise
	HRESULT	hr = S_OK;
	m_whichBase = Uk;
	initialiseHydroOutputs();
	
	//Set input data
	m_diameter = diameter;
	m_roughness = roughness;
	m_slopeSinus = slopeSinus;
	m_nbrOfPresentPhases = nbrOfPresentPhases;

	for(int i=0; i<nbrOfPresentPhases; ++i){
		m_phaseQualifiers[i] = phaseQualifiers[i];
		int indice = phaseQualifiers[i];
		m_superficialVelocities[indice] = superficialVelocities[indice];
		m_densities[indice] = densities[indice];
		m_viscosities[indice] = viscosities[indice];
		m_heatCapacities[indice] = heatCapacities[indice];
		m_conductivities[indice] = conductivities[indice];
		for(int j=0; j<nbrOfPresentPhases; ++j){
			int indice2 = phaseQualifiers[j];
			m_surfaceTensions[indice][indice2] = surfaceTensions[indice][indice2];
		}
	}

	// Computation
	int indice;
	m_mixtureVelocity = 0.;
	for(int i = 0; i<m_nbrOfPresentPhases; ++i){
		indice = m_phaseQualifiers[i];
		m_mixtureVelocity += m_superficialVelocities[indice];
	}
	//double dVGL = 0.35e0*sqrt(gravity*m_diameter)*m_slopeSinus;
	//CMG
	double dVGL = (0.2*m_mixtureVelocity+0.35e0*sqrt(gravity*m_diameter))*m_slopeSinus;

	m_barycentricVelocity = 0.;
	double denominateurVelocity = 0.;
	
	//WARN: OIL and WATER absolute velocities need GAS absolute velocity: so if gas phase is not present, other velocities are equal to m_mixtureVelocity
	double gasAbsoluteVelocity = (! isNaN(m_superficialVelocities[GAS]))? (m_mixtureVelocity + dVGL + sgnVal(m_mixtureVelocity) * sqrt(pow(m_mixtureVelocity+dVGL,2)-4e0*m_superficialVelocities[GAS]*dVGL))/2e0 : m_mixtureVelocity + dVGL;

	double sum = 0.;
	for(int i=0; i<m_nbrOfPresentPhases; ++i){
		indice = m_phaseQualifiers[i];
		if(indice != GAS)
			m_absoluteVelocities[indice] = gasAbsoluteVelocity - dVGL;
		else
			m_absoluteVelocities[GAS] = gasAbsoluteVelocity; //DO not need to test if gasAbsoluteVelocity is equal to DOUBLE_INF, if it is, gas phase is not present!!!!

		if(i == m_nbrOfPresentPhases - 1)
			m_localVolumetricFractions[indice] = 1.-sum;
		else
			sum += m_localVolumetricFractions[indice] = m_superficialVelocities[indice]/m_absoluteVelocities[indice];
		
		m_flowingVolumetricFractions[indice] = m_superficialVelocities[indice]/m_mixtureVelocity;
		m_barycentricVelocity += m_densities[indice] * m_superficialVelocities[indice];
		denominateurVelocity += m_densities[indice] * m_localVolumetricFractions[indice];
	}

	m_barycentricVelocity /= denominateurVelocity;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
HRESULT CSimpleHydro::hydroRkVm(const double diameter,const double roughness,const double slopeSinus,const int nbrOfPresentPhases,
								const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &localVolumetricFractions,
								double barycentricVelocity,
								const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
								const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
								const TNT::Array1D<double> &conductivities) {

	// Initialise
	HRESULT	hr = S_OK;
	m_whichBase = RkVm;
	initialiseHydroOutputs();
	
	// Set input data
	m_diameter           = diameter;
	m_slopeSinus         = slopeSinus;
	m_roughness          = roughness;
	m_nbrOfPresentPhases = nbrOfPresentPhases;
	m_phaseQualifiers    = phaseQualifiers.copy();
	m_densities          = densities.copy();
	m_viscosities        = viscosities.copy();
	m_surfaceTensions    = surfaceTensions.copy();
	m_heatCapacities     = heatCapacities.copy();
	m_conductivities     = conductivities.copy();

	m_localVolumetricFractions = localVolumetricFractions.copy();
	m_barycentricVelocity      = barycentricVelocity;

	// Computation
	double rhoM = m_densities[GAS]*m_localVolumetricFractions[GAS] + m_densities[OIL]*m_localVolumetricFractions[OIL] + m_densities[WATER]*m_localVolumetricFractions[WATER];
	double dVGL = 0.35e0*sqrt(gravity*m_diameter)*m_slopeSinus;

	m_absoluteVelocities[GAS]   = m_barycentricVelocity + dVGL*(m_densities[OIL]*m_localVolumetricFractions[OIL]+m_densities[WATER]*m_localVolumetricFractions[WATER])/rhoM;
	m_absoluteVelocities[OIL]   = m_absoluteVelocities[GAS] - dVGL;
	m_absoluteVelocities[WATER] = m_absoluteVelocities[GAS] - dVGL;

	m_superficialVelocities[GAS]   = m_localVolumetricFractions[GAS]*m_absoluteVelocities[GAS];
	m_superficialVelocities[OIL]   = m_localVolumetricFractions[OIL]*m_absoluteVelocities[OIL];
	m_superficialVelocities[WATER] = m_localVolumetricFractions[WATER]*m_absoluteVelocities[WATER];
	m_mixtureVelocity              = m_superficialVelocities[GAS] + m_superficialVelocities[OIL] + m_superficialVelocities[WATER];

	m_flowingVolumetricFractions[GAS]   = m_superficialVelocities[GAS]/m_mixtureVelocity;
	m_flowingVolumetricFractions[OIL]   = m_superficialVelocities[OIL]/m_mixtureVelocity;
	m_flowingVolumetricFractions[WATER] = m_superficialVelocities[WATER]/m_mixtureVelocity;

	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
HRESULT CSimpleHydro::hydroRkUm(const double diameter, const double roughness, const double slopeSinus,
					  const int nbrOfPresentPhases, const TNT::Array1D<EPhases> &phaseQualifiers,
					  const TNT::Array1D<double> &volumetricFractions, const double          mixtureVelocity,
					  const TNT::Array1D<double> &densities,           const TNT::Array1D<double> &viscosities,
					  const TNT::Array2D<double> &surfaceTensions,     const TNT::Array1D<double> &heatCapacities,
					  const TNT::Array1D<double> &conductivities)
{
	// Initialise
	HRESULT	hr = S_OK;
	m_whichBase = RkUm;
	initialiseHydroOutputs();
	const double FACTOR_DV = 0.2;
	
	// Set input data
	m_diameter           = diameter;
	m_slopeSinus         = slopeSinus;
	m_roughness          = roughness;
	m_nbrOfPresentPhases = nbrOfPresentPhases;
	m_phaseQualifiers    = phaseQualifiers.copy();
	m_densities          = densities.copy();
	m_viscosities        = viscosities.copy();
	m_surfaceTensions    = surfaceTensions.copy();
	m_heatCapacities     = heatCapacities.copy();
	m_conductivities     = conductivities.copy();

	m_localVolumetricFractions = volumetricFractions.copy();
	m_mixtureVelocity		   = mixtureVelocity;

	// Computation
	double rhoM = m_densities[GAS]*m_localVolumetricFractions[GAS] + m_densities[OIL]*m_localVolumetricFractions[OIL] + m_densities[WATER]*m_localVolumetricFractions[WATER];
	double dVGL = (FACTOR_DV * m_mixtureVelocity + 0.35e0*sqrt(gravity*m_diameter)) * m_slopeSinus;

	m_absoluteVelocities[GAS]   = m_mixtureVelocity + dVGL * (m_localVolumetricFractions[OIL] + m_localVolumetricFractions[WATER]);
	m_absoluteVelocities[OIL]   = m_mixtureVelocity - dVGL * m_localVolumetricFractions[GAS];
	m_absoluteVelocities[WATER] = m_absoluteVelocities[OIL];

	m_superficialVelocities[GAS]   = m_localVolumetricFractions[GAS] * m_absoluteVelocities[GAS];
	m_superficialVelocities[OIL]   = m_localVolumetricFractions[OIL] * m_absoluteVelocities[OIL];
	m_superficialVelocities[WATER] = m_localVolumetricFractions[WATER] * m_absoluteVelocities[WATER];

	m_flowingVolumetricFractions[GAS]   = m_superficialVelocities[GAS]/m_mixtureVelocity;
	m_flowingVolumetricFractions[OIL]   = m_superficialVelocities[OIL]/m_mixtureVelocity;
	m_flowingVolumetricFractions[WATER] = m_superficialVelocities[WATER]/m_mixtureVelocity;

	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
HRESULT CSimpleHydro::computeProperties(){
	// Initialise
	HRESULT	hr  = S_OK;
	double rhoM = 0.;
	for(int i = 0; i<m_nbrOfPresentPhases; ++i){
		int indice = m_phaseQualifiers[i];
		rhoM += m_densities[indice]*m_localVolumetricFractions[indice];
	}

	// Compute flow parameters
	//hr = checkPresentPhases();

	unsigned int value2find = 0;
	for(int i=0; i<m_nbrOfPresentPhases; ++i)
		value2find += (unsigned int)pow(2., m_phaseQualifiers[i]);

	std::map<unsigned int, std::pair<EHydroFlowPatterns, double> >::iterator it = m_mapHydroProp.find(value2find);

	R2PException::Utils::ThrowException(it != m_mapHydroProp.end(), R2PException::R2P_EBADARG, L"Invalid hydro configuration.", L"Fail to compute hydro properties: present phases are not valid. Please contact support.\n");

	m_flowPattern = it->second.first;
	m_slugFraction = it->second.second;



	// Compute local pressure drop
	m_frictionPressureDrop = -2e0*rhoM*m_frictionFactor*m_mixtureVelocity*absVal(m_mixtureVelocity)/m_diameter;
	m_gravityPressureDrop = - rhoM*gravity*m_slopeSinus;
	m_localPressureDrop = m_frictionPressureDrop + m_gravityPressureDrop;


	// Compute heat resistivity
	//tmpVal allows to compute Re and Pr for all present phases
	double tmpVal[2] = {0., 0.};
	double tmpInternalFluidResistivity = 0.;
	for(int i=0; i<m_nbrOfPresentPhases; ++i){
		int indice = m_phaseQualifiers[i];
		tmpVal[0] = m_densities[indice]*m_diameter*absVal(m_superficialVelocities[indice])/max(m_viscosities[indice],MIN_VISCOSITIES[indice]);
		tmpVal[1] = m_viscosities[indice]*m_heatCapacities[indice]/max(m_conductivities[indice],MIN_CONDUCTIVITY[indice]);
		tmpInternalFluidResistivity += m_localVolumetricFractions[indice] * m_conductivities[indice] * nusselt(tmpVal[0], tmpVal[1]);
	}
	m_internalFluidResistivity = 1e0/PI/tmpInternalFluidResistivity;

	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
HRESULT CSimpleHydro::computeDerivatives()
{
	HRESULT	hr;

	switch (m_whichBase)
	{
	case Uk :
		hr = S_FALSE;
		break;

	case RkVm :
		hr = S_OK;
		computeDerivativesRkVm();
		break;

	case RkUm :
		hr = S_OK;
		computeDerivativesRkUm();
		break;
	}
	return hr;
	
}

/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
void CSimpleHydro::computeDerivativesRkVm()
{
	// Initialise
	double rhoM = m_densities[GAS]*m_localVolumetricFractions[GAS] + m_densities[OIL]*m_localVolumetricFractions[OIL] + m_densities[WATER]*m_localVolumetricFractions[WATER];
	double dVGL = 0.35e0*sqrt(gravity*m_diameter)*m_slopeSinus;

	// Compute the "all phases" absolute velocity derivative
	double dvdrG   = m_densities[GAS]*(m_densities[WATER]*(m_localVolumetricFractions[OIL]-1e0)-m_densities[OIL]*m_localVolumetricFractions[OIL])*dVGL/rhoM/rhoM;
	double dvdrO   = m_densities[GAS]*m_localVolumetricFractions[GAS]*(m_densities[OIL]-m_densities[WATER])*dVGL/rhoM/rhoM;
	double dvdrhoG = -m_localVolumetricFractions[GAS]*(m_densities[OIL]*m_localVolumetricFractions[OIL]+m_densities[WATER]*m_localVolumetricFractions[WATER])*dVGL/rhoM/rhoM;
	double dvdrhoO = m_densities[GAS]*m_localVolumetricFractions[GAS]*m_localVolumetricFractions[OIL]*dVGL/rhoM/rhoM;
	double dvdrhoW = m_densities[GAS]*m_localVolumetricFractions[GAS]*m_localVolumetricFractions[WATER]*dVGL/rhoM/rhoM;

	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][GAS] = dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][OIL] = dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][WATER] = 0.;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][GAS] = dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][OIL] = dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][WATER] = 0.;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][GAS] = dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][OIL] = dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][WATER] = 0.;
	m_absoluteVelocitiesDBarycentricVelocity[GAS] = 1e0;
	m_absoluteVelocitiesDBarycentricVelocity[OIL] = 1e0;
	m_absoluteVelocitiesDBarycentricVelocity[WATER] = 1e0;
	m_absoluteVelocitiesDMixtureVelocity[GAS] = 1e0;
	m_absoluteVelocitiesDMixtureVelocity[OIL] = 1e0;
	m_absoluteVelocitiesDMixtureVelocity[WATER] = 1e0;
	m_absoluteVelocitiesDDensities[GAS][GAS] = dvdrhoG;
	m_absoluteVelocitiesDDensities[GAS][OIL] = dvdrhoO;
	m_absoluteVelocitiesDDensities[GAS][WATER] = dvdrhoW;
	m_absoluteVelocitiesDDensities[OIL][GAS] = dvdrhoG;
	m_absoluteVelocitiesDDensities[OIL][OIL] = dvdrhoO;
	m_absoluteVelocitiesDDensities[OIL][WATER] = dvdrhoW;
	m_absoluteVelocitiesDDensities[WATER][GAS] = dvdrhoG;
	m_absoluteVelocitiesDDensities[WATER][OIL] = dvdrhoO;
	m_absoluteVelocitiesDDensities[WATER][WATER] = dvdrhoW;
	m_absoluteVelocitiesDViscosities[GAS][GAS] = 0e0;
	m_absoluteVelocitiesDViscosities[GAS][OIL] = 0e0;
	m_absoluteVelocitiesDViscosities[GAS][WATER] = 0e0;
	m_absoluteVelocitiesDViscosities[OIL][GAS] = 0e0;
	m_absoluteVelocitiesDViscosities[OIL][OIL] = 0e0;
	m_absoluteVelocitiesDViscosities[OIL][WATER] = 0e0;
	m_absoluteVelocitiesDViscosities[WATER][GAS] = 0e0;
	m_absoluteVelocitiesDViscosities[WATER][OIL] = 0e0;
	m_absoluteVelocitiesDViscosities[WATER][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][WATER] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][GAS] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][OIL] = 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][WATER] = 0e0;

	// Compute the local pressure drop derivative
	double coefA = 2e0*m_frictionFactor*m_mixtureVelocity*absVal(m_mixtureVelocity)/m_diameter + gravity*m_slopeSinus; // = -dPds/rhoM
	double coefB = 4e0*rhoM*m_frictionFactor*absVal(m_mixtureVelocity)/m_diameter;

	m_localPressureDropDLocalVolumetricFractions[GAS] = coefA*(m_densities[WATER]-m_densities[GAS]) - coefB*(dvdrG+dVGL);
	m_localPressureDropDLocalVolumetricFractions[OIL] = coefA*(m_densities[WATER]-m_densities[OIL]) - coefB*dvdrO;
	m_localPressureDropDLocalVolumetricFractions[WATER] = 0e0;
	m_localPressureDropDBarycentricVelocity             = -coefB;
	m_localPressureDropDDensities[GAS]                = -coefA*m_localVolumetricFractions[GAS] - coefB*dvdrhoG;
	m_localPressureDropDDensities[OIL]                = -coefA*m_localVolumetricFractions[OIL] - coefB*dvdrhoO;
	m_localPressureDropDDensities[WATER]                = -coefA*m_localVolumetricFractions[WATER] - coefB*dvdrhoW;
	m_localPressureDropDViscosities[GAS]              = 0e0;
	m_localPressureDropDViscosities[OIL]              = 0e0;
	m_localPressureDropDViscosities[WATER]              = 0e0;
	m_localPressureDropDSurfaceTensions[GAS][GAS]   = 0e0;
	m_localPressureDropDSurfaceTensions[OIL][GAS]   = 0e0;
	m_localPressureDropDSurfaceTensions[WATER][GAS] = 0e0;
	m_localPressureDropDSurfaceTensions[GAS][OIL]   = 0e0;
	m_localPressureDropDSurfaceTensions[OIL][OIL]   = 0e0;
	m_localPressureDropDSurfaceTensions[WATER][OIL] = 0e0;
	m_localPressureDropDSurfaceTensions[GAS][WATER]   = 0e0;
	m_localPressureDropDSurfaceTensions[OIL][WATER]   = 0e0;
	m_localPressureDropDSurfaceTensions[WATER][WATER] = 0e0;
	
	// Compute the internal fluid resistivity derivative
	double dNusltdReG, dNusltdPrG;
	double ReG    = m_densities[GAS]*m_diameter*absVal(m_superficialVelocities[GAS])/max(m_viscosities[GAS],MIN_VISCOSITIES_GAS);
	double PrG    = m_viscosities[GAS]*m_heatCapacities[GAS]/max(m_conductivities[GAS],MIN_CONDUCTIVITY_GAS);
	double NusltG = derNusselt(ReG, PrG, &dNusltdReG, &dNusltdPrG);
	double dNusltdReO, dNusltdPrO;
	double ReO    = m_densities[OIL]*m_diameter*absVal(m_superficialVelocities[OIL])/max(m_viscosities[OIL],MIN_VISCOSITIES_OIL);
	double PrO    = m_viscosities[OIL]*m_heatCapacities[OIL]/max(m_conductivities[OIL],MIN_CONDUCTIVITY_OIL);
	double NusltO = derNusselt(ReO, PrO, &dNusltdReO, &dNusltdPrO);
	double dNusltdReW, dNusltdPrW;
	double ReW    = m_densities[WATER]*m_diameter*absVal(m_superficialVelocities[WATER])/max(m_viscosities[WATER],MIN_VISCOSITIES_WATER );
	double PrW    = m_viscosities[WATER]*m_heatCapacities[WATER]/max(m_conductivities[WATER],MIN_CONDUCTIVITY_WATER);
	double NusltW = derNusselt(ReW, PrW, &dNusltdReW, &dNusltdPrW);
	TNT::Array1D<double> kinematicViscosities(NBPHASES);
	kinematicViscosities[GAS]	= MIN_VISCOSITIES_GAS;
	kinematicViscosities[OIL]	= MIN_VISCOSITIES_OIL;
	kinematicViscosities[WATER]	= MIN_VISCOSITIES_WATER;
	for (int i = 0; i<NBPHASES;i++)
		if (m_densities[i] != 0.) kinematicViscosities[i] /= m_densities[i];

	m_internalFluidResistivityDLocalVolumetricFractions[GAS] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_conductivities[GAS]*NusltG - m_conductivities[WATER]*NusltW
		+m_diameter*m_localVolumetricFractions[OIL]*m_localVolumetricFractions[OIL]*m_conductivities[OIL]/kinematicViscosities[OIL]*dNusltdReO*dvdrG*sgnVal(m_absoluteVelocities[OIL])
		+m_diameter*m_localVolumetricFractions[GAS]*m_conductivities[GAS]/kinematicViscosities[GAS]*dNusltdReG*(m_localVolumetricFractions[GAS]*dvdrG*sgnVal(m_absoluteVelocities[GAS]) + absVal(m_absoluteVelocities[GAS]))
		+m_diameter*m_localVolumetricFractions[WATER]*m_conductivities[WATER]/kinematicViscosities[WATER]*dNusltdReW*(m_localVolumetricFractions[WATER]*dvdrG*sgnVal(m_absoluteVelocities[WATER]) - absVal(m_absoluteVelocities[WATER])));
	m_internalFluidResistivityDLocalVolumetricFractions[OIL] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_conductivities[OIL]*NusltO - m_conductivities[WATER]*NusltW
		+m_diameter*m_localVolumetricFractions[GAS]*m_localVolumetricFractions[GAS]*m_conductivities[GAS]/kinematicViscosities[GAS]*dNusltdReG*dvdrO*sgnVal(m_absoluteVelocities[GAS])
		+m_diameter*m_localVolumetricFractions[OIL]*m_conductivities[OIL]/kinematicViscosities[OIL]*dNusltdReO*(m_localVolumetricFractions[OIL]*dvdrO*sgnVal(m_absoluteVelocities[OIL]) + absVal(m_absoluteVelocities[OIL]))
		+m_diameter*m_localVolumetricFractions[WATER]*m_conductivities[WATER]/kinematicViscosities[WATER]*dNusltdReW*(m_localVolumetricFractions[WATER]*dvdrO*sgnVal(m_absoluteVelocities[WATER]) - absVal(m_absoluteVelocities[WATER])));
	m_internalFluidResistivityDLocalVolumetricFractions[WATER] = 0e0;
	m_internalFluidResistivityDBarycentricVelocity = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_diameter*(m_localVolumetricFractions[GAS]*m_localVolumetricFractions[GAS]*m_conductivities[GAS]/kinematicViscosities[GAS]*dNusltdReG*sgnVal(m_absoluteVelocities[GAS])
		+m_localVolumetricFractions[OIL]*m_localVolumetricFractions[OIL]*m_conductivities[OIL]/kinematicViscosities[OIL]*dNusltdReO*sgnVal(m_absoluteVelocities[OIL])
		+m_localVolumetricFractions[WATER]*m_localVolumetricFractions[WATER]*m_conductivities[WATER]/kinematicViscosities[WATER]*dNusltdReW*sgnVal(m_absoluteVelocities[WATER]));
	m_internalFluidResistivityDDensities[GAS] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_localVolumetricFractions[GAS]*m_conductivities[GAS]*(m_diameter*m_localVolumetricFractions[GAS]/kinematicViscosities[GAS]*dNusltdReG*sgnVal(m_absoluteVelocities[GAS])*dvdrhoG+kinematicViscosities[GAS]*m_heatCapacities[GAS]/m_conductivities[GAS]*dNusltdPrG)
		+m_diameter*m_localVolumetricFractions[OIL]*m_localVolumetricFractions[OIL]*m_conductivities[OIL]/kinematicViscosities[OIL]*dNusltdReO*sgnVal(m_absoluteVelocities[OIL])*dvdrhoG
		+m_diameter*m_localVolumetricFractions[WATER]*m_localVolumetricFractions[WATER]*m_conductivities[WATER]/kinematicViscosities[WATER]*dNusltdReW*sgnVal(m_absoluteVelocities[WATER])*dvdrhoG);
	m_internalFluidResistivityDDensities[OIL] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_diameter*m_localVolumetricFractions[GAS]*m_localVolumetricFractions[GAS]*m_conductivities[GAS]/kinematicViscosities[GAS]*dNusltdReG*sgnVal(m_absoluteVelocities[GAS])*dvdrhoO
		+m_localVolumetricFractions[OIL]*m_conductivities[OIL]*(m_diameter*m_localVolumetricFractions[OIL]/kinematicViscosities[OIL]*dNusltdReO*sgnVal(m_absoluteVelocities[OIL])*dvdrhoO+kinematicViscosities[OIL]*m_heatCapacities[OIL]/m_conductivities[OIL]*dNusltdPrO)
		+m_diameter*m_localVolumetricFractions[WATER]*m_localVolumetricFractions[WATER]*m_conductivities[WATER]/kinematicViscosities[WATER]*dNusltdReW*sgnVal(m_absoluteVelocities[WATER])*dvdrhoO);
	m_internalFluidResistivityDDensities[WATER] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_diameter*m_localVolumetricFractions[GAS]*m_localVolumetricFractions[GAS]*m_conductivities[GAS]/kinematicViscosities[GAS]*dNusltdReG*sgnVal(m_absoluteVelocities[GAS])*dvdrhoW
		+m_diameter*m_localVolumetricFractions[OIL]*m_localVolumetricFractions[OIL]*m_conductivities[OIL]/kinematicViscosities[OIL]*dNusltdReO*sgnVal(m_absoluteVelocities[OIL])*dvdrhoW
		+m_localVolumetricFractions[WATER]*m_conductivities[WATER]*(m_diameter*m_localVolumetricFractions[WATER]/kinematicViscosities[WATER]*dNusltdReW*sgnVal(m_absoluteVelocities[WATER])*dvdrhoW+kinematicViscosities[WATER]*m_heatCapacities[WATER]/m_conductivities[WATER]*dNusltdPrW));
	m_internalFluidResistivityDViscosities[GAS] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[GAS]*m_conductivities[GAS]*(m_densities[GAS]*m_heatCapacities[GAS]/m_conductivities[GAS]*dNusltdPrG - ReG*dNusltdReG/kinematicViscosities[GAS]);
	m_internalFluidResistivityDViscosities[OIL] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[OIL]*m_conductivities[OIL]*(m_densities[OIL]*m_heatCapacities[OIL]/m_conductivities[OIL]*dNusltdPrO - ReO*dNusltdReO/kinematicViscosities[OIL]);
	m_internalFluidResistivityDViscosities[WATER] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[WATER]*m_conductivities[WATER]*(m_densities[WATER]*m_heatCapacities[WATER]/m_conductivities[WATER]*dNusltdPrW - ReW*dNusltdReW/kinematicViscosities[WATER]);
	m_internalFluidResistivityDSurfaceTensions[GAS][GAS] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[OIL][GAS] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[WATER][GAS] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[GAS][OIL] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[OIL][OIL] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[WATER][OIL] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[GAS][WATER] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[OIL][WATER] = 0e0;
	m_internalFluidResistivityDSurfaceTensions[WATER][WATER] = 0e0;
	m_internalFluidResistivityDConductivities[GAS] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_localVolumetricFractions[GAS]*NusltG - m_localVolumetricFractions[GAS]*m_densities[GAS]*kinematicViscosities[GAS]*m_heatCapacities[GAS]/m_conductivities[GAS]*dNusltdPrG);
	m_internalFluidResistivityDConductivities[OIL] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_localVolumetricFractions[OIL]*NusltO - m_localVolumetricFractions[OIL]*m_densities[OIL]*kinematicViscosities[OIL]*m_heatCapacities[OIL]/m_conductivities[OIL]*dNusltdPrO);
	m_internalFluidResistivityDConductivities[WATER] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*(m_localVolumetricFractions[WATER]*NusltW - m_localVolumetricFractions[WATER]*m_densities[WATER]*kinematicViscosities[WATER]*m_heatCapacities[WATER]/m_conductivities[WATER]*dNusltdPrW);
	m_internalFluidResistivityDHeatCapacities[GAS] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[GAS]*m_densities[GAS]*kinematicViscosities[GAS]*dNusltdPrG;
	m_internalFluidResistivityDHeatCapacities[OIL] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[OIL]*m_densities[OIL]*kinematicViscosities[OIL]*dNusltdPrO;
	m_internalFluidResistivityDHeatCapacities[WATER] = -PI*m_internalFluidResistivity*m_internalFluidResistivity*m_localVolumetricFractions[WATER]*m_densities[WATER]*kinematicViscosities[WATER]*dNusltdPrW;

}


/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
void CSimpleHydro::computeDerivativesRkUm()
{
	// Initialise
	const double FACTOR_DV = 0.2;
	double rhoM = m_densities[GAS]*m_localVolumetricFractions[GAS] + m_densities[OIL]*m_localVolumetricFractions[OIL] + m_densities[WATER]*m_localVolumetricFractions[WATER];
	double dVGL = (FACTOR_DV * m_mixtureVelocity + 0.35e0*sqrt(gravity*m_diameter)) * m_slopeSinus;

	// Compute the "all phases" absolute velocity derivative
	double dvdrG   = -dVGL;
	double dvdrO   = 0.;
	double dvdrhoG = 0.;
	double dvdrhoO = 0.;
	double dvdrhoW = 0.;

	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][GAS]		= dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][OIL]		= dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[GAS][WATER]	= 0.;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][GAS]		= dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][OIL]		= dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[OIL][WATER]	= 0.;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][GAS]	= dvdrG;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][OIL]	= dvdrO;
	m_absoluteVelocitiesDLocalVolumetricFractions[WATER][WATER]	= 0.;

	m_absoluteVelocitiesDMixtureVelocity[GAS]	= 1. + FACTOR_DV * m_slopeSinus * (m_localVolumetricFractions[OIL] + m_localVolumetricFractions[WATER]);
	m_absoluteVelocitiesDMixtureVelocity[OIL]	= 1. - FACTOR_DV * m_slopeSinus * m_localVolumetricFractions[GAS];
	m_absoluteVelocitiesDMixtureVelocity[WATER]	= 1. - FACTOR_DV * m_slopeSinus * m_localVolumetricFractions[GAS];

	m_absoluteVelocitiesDDensities[GAS][GAS]		= dvdrhoG;
	m_absoluteVelocitiesDDensities[GAS][OIL]		= dvdrhoO;
	m_absoluteVelocitiesDDensities[GAS][WATER]		= dvdrhoW;
	m_absoluteVelocitiesDDensities[OIL][GAS]		= dvdrhoG;
	m_absoluteVelocitiesDDensities[OIL][OIL]		= dvdrhoO;
	m_absoluteVelocitiesDDensities[OIL][WATER]		= dvdrhoW;
	m_absoluteVelocitiesDDensities[WATER][GAS]		= dvdrhoG;
	m_absoluteVelocitiesDDensities[WATER][OIL]		= dvdrhoO;
	m_absoluteVelocitiesDDensities[WATER][WATER]	= dvdrhoW;

	m_absoluteVelocitiesDViscosities[GAS][GAS]		= 0e0;
	m_absoluteVelocitiesDViscosities[GAS][OIL]		= 0e0;
	m_absoluteVelocitiesDViscosities[GAS][WATER]	= 0e0;
	m_absoluteVelocitiesDViscosities[OIL][GAS]		= 0e0;
	m_absoluteVelocitiesDViscosities[OIL][OIL]		= 0e0;
	m_absoluteVelocitiesDViscosities[OIL][WATER]	= 0e0;
	m_absoluteVelocitiesDViscosities[WATER][GAS]	= 0e0;
	m_absoluteVelocitiesDViscosities[WATER][OIL]	= 0e0;
	m_absoluteVelocitiesDViscosities[WATER][WATER]	= 0e0;

	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][GAS]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][OIL]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][GAS][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][GAS]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][OIL]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][OIL][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][GAS]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][OIL]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[GAS][WATER][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][GAS]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][OIL]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][GAS][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][GAS]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][OIL]			= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][OIL][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][GAS]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][OIL]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[OIL][WATER][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][GAS]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][OIL]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][GAS][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][GAS]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][OIL]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][OIL][WATER]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][GAS]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][OIL]		= 0e0;
	m_absoluteVelocitiesDSurfaceTensions[WATER][WATER][WATER]	= 0e0;

	// Compute the local pressure drop derivatives
	double coefA = 2e0*m_frictionFactor*m_mixtureVelocity*absVal(m_mixtureVelocity)/m_diameter + gravity*m_slopeSinus; // = -dPds/rhoM
	double coefB = 4e0*rhoM*m_frictionFactor*absVal(m_mixtureVelocity)/m_diameter;

	m_localPressureDropDLocalVolumetricFractions[GAS]	= coefA*(m_densities[WATER]-m_densities[GAS]);
	m_localPressureDropDLocalVolumetricFractions[OIL]	= coefA*(m_densities[WATER]-m_densities[OIL]);
	m_localPressureDropDLocalVolumetricFractions[WATER] = 0.;
	m_localPressureDropDMixtureVelocity		            = -coefB;
	m_localPressureDropDDensities[GAS]					= -coefA * m_localVolumetricFractions[GAS];
	m_localPressureDropDDensities[OIL]					= -coefA * m_localVolumetricFractions[OIL];
	m_localPressureDropDDensities[WATER]                = -coefA * m_localVolumetricFractions[WATER];
	m_localPressureDropDViscosities[GAS]				= 0e0;
	m_localPressureDropDViscosities[OIL]				= 0e0;
	m_localPressureDropDViscosities[WATER]              = 0e0;
	m_localPressureDropDSurfaceTensions[GAS][GAS]		= 0e0;
	m_localPressureDropDSurfaceTensions[OIL][GAS]		= 0e0;
	m_localPressureDropDSurfaceTensions[WATER][GAS]		= 0e0;
	m_localPressureDropDSurfaceTensions[GAS][OIL]		= 0e0;
	m_localPressureDropDSurfaceTensions[OIL][OIL]		= 0e0;
	m_localPressureDropDSurfaceTensions[WATER][OIL]		= 0e0;
	m_localPressureDropDSurfaceTensions[GAS][WATER]		= 0e0;
	m_localPressureDropDSurfaceTensions[OIL][WATER]		= 0e0;
	m_localPressureDropDSurfaceTensions[WATER][WATER]	= 0e0;
	
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CSimpleHydro::absVal(double x) const
{
	return x>=0e0 ? x : -x;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CSimpleHydro::sgnVal(double x) const
{
	return x>=0e0 ? 1e0 : -1e0;
}
/****************************************************************************************************************************
*/
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CSimpleHydro::nusselt(double phaseReynold,double  phasePrandtl) const
{
	// Turbulent flow: Stanton correlation in Nusselt form
	double TUR_STANTON = 0.023;
	double turbulentNu = TUR_STANTON * pow(fabs(phaseReynold),0.8) * pow(fabs(phasePrandtl),0.333333333333);

	// Laminar flow: Nusselt number
    double laminarNu = 4.;

	// Choose maximum Nusselt number
	return max(laminarNu,turbulentNu);
}

/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
/** 

*/
double CSimpleHydro::derNusselt(double phaseReynolds,double  phasePrandtl,double  *nuDPhaseReynolds,double  *nuDPhasePrandtl) const
{
	// Turbulent flow: Stanton correlation in Nusselt form
	double TUR_STANTON = 0.023;
	double turbulentNu = TUR_STANTON * pow(fabs(phaseReynolds),0.8) * pow(fabs(phasePrandtl),0.333333333333);
	double turbulentNuDPhaseReynolds = 0.8*turbulentNu/fabs(phaseReynolds);
    double turbulentNuDPhasePrandtl = turbulentNu/(fabs(phasePrandtl)*3.);

	// Laminar flow: Nusselt number
    double laminarNu = 4.;
	double laminarNuDPhaseReynolds = 0.;
    double laminarNuDPhasePrandtl = 0.;

	// Choose maximum Nusselt number

	if (laminarNu > turbulentNu)
	{
		*nuDPhaseReynolds = laminarNuDPhaseReynolds;
		*nuDPhasePrandtl = laminarNuDPhasePrandtl;
		return laminarNu;
	}
	else
	{
		*nuDPhaseReynolds = turbulentNuDPhaseReynolds;
		*nuDPhasePrandtl = turbulentNuDPhasePrandtl;
		return turbulentNu;
	}
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
HRESULT CSimpleHydro::checkPresentPhases(){
	HRESULT	hr			=	S_OK;
	
	for (int i = 0; i<NBPHASES; i++)
		m_isPresentPhases[i] = false;


	for (int i = 0; i<NBPHASES; i++)
		if(m_localVolumetricFractions[i] > 1.e-10) m_isPresentPhases[i] = true;


	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
enum hydroCst::modelHydro CSimpleHydro::get_hydroID(){
	return hydroCst::SIMP;
}
