#include "CCapeBadCOParameter.h"

namespace R2PException{
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeBadCOParameter::CCapeBadCOParameter(HRESULT hr, const std::wstring& desc): CCapeData(hr, desc){
	_parameterName = L"";
	_parameter = NULL;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeBadCOParameter::CCapeBadCOParameter(IDispatch* pDisp): CCapeData(pDisp){
	ECapeBadCOParameter* pCape = NULL;
	HRESULT hres = pDisp->QueryInterface(IID_ECapeBadCOParameter, (void**)&pCape);
	if(FAILED(hres)){
		_description += L"WARNING: Cannot access ECapeBadCOParameter error interface from a third CAPE_OPEN component: attributes are undefined.\n";
		_parameterName = L"";
		_parameter = NULL;
	}
	else{
		BSTR bstr_parameterName = NULL;
		hres = pCape->get_parameter(&_parameter);
		if(FAILED(hres) || _parameter == NULL){
			_description += L"WARNING: Failed while getting parameter: value is undefined.\n";
			_parameter = NULL;
		}
		hres = pCape->get_parameterName(&bstr_parameterName);
		if(SUCCEEDED(hres) && bstr_parameterName != NULL)
			_parameterName = bstr_parameterName;
		else{
			_description += L"WARNING: Failed while getting parameterName: value is undefined.\n";
			_parameterName = L"";
		}
		::SysFreeString(bstr_parameterName);
	}
	if(pCape) pCape->Release();
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
CCapeBadCOParameter::~CCapeBadCOParameter(){
	if(_parameter)
		_parameter->Release();
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeBadCOParameter::set_parameterName(const std::wstring& ParamName){
	_parameterName = ParamName;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeBadCOParameter::set_parameter(const IDispatch* theParam){
	_parameter = const_cast<IDispatch*>(theParam);
	if(_parameter) _parameter->AddRef();
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeBadCOParameter::get_parameterName(std::wstring& ParamName){
	ParamName = _parameterName;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/ 
void CCapeBadCOParameter::get_parameter(IDispatch** theParam){
	*theParam = _parameter;
	if((*theParam)) (*theParam)->AddRef();
}

} //End namespace R2PException
