// CBadParamteter.h: interface for the ECapeBadCOParameter class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEBOUDARIES_H__)
#define __CCAPEBOUDARIES_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: A "utility" class which factorises a state which describes the value, its type and its boundaries</summary> 
	class CCapeBoundaries
	{
	public:
		/// <summary>Constructor</summary>
		CCapeBoundaries();

		/// <summary>Constructor: create a CCapeBoundaries object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeBoundaries interface</param>
		CCapeBoundaries(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		virtual ~CCapeBoundaries() = 0;

		/// <summary>set error specific class attribute</summary>
		/// <param name="lowerBound">lower bound value to set</param>
		/// <param name="upperBound">upper bound value to set</param>
		/// <param name="value">current value to set</param>
		/// <param name="type">type to set</param>
		/// <returns>void</returns>
		void set_boundaries(double lbound, double ubound, double value, const std::wstring& type);

		/// <summary>get error specific class attribute</summary>
		/// <param name="lowerBound">lower bound value to set</param>
		/// <returns>void</returns>
		void get_LowerBound(double& lbound);

		/// <summary>get error specific class attribute</summary>
		/// <param name="upperBound">upper bound value to set</param>
		/// <returns>void</returns>
		void get_UpperBound(double& ubound);

		/// <summary>get error specific class attribute</summary>
		/// <param name="value">current value to set</param>
		/// <returns>void</returns>
		void get_value(double& value);

		/// <summary>get error specific class attribute</summary>
		/// <param name="type">type to set</param>
		/// <returns>void</returns>
		void get_Type(std::wstring& type);

	protected:
		/// <summary>The value of the lower bound</summary>
		double _lowerBound;
		/// <summary>The value of the upper bound</summary>
		double _upperBound;
		/// <summary>The current value which has led to an error</summary>
		double _value;
		/// <summary>The type/nature of the value. The value could represent a thermodynamic property, a number of tables in a database, a quantity of memory, ...</summary>
		std::wstring _type;
	};
}
#endif // __CBOUDARIES_H__
