// Exception.cpp: implementation of the CMyException class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CMyException.h"
#include "ExceptionUtils.h"

#include "CCapeBadArgument.h"
#include "CCapeBadCOParameter.h"
#include "CCapeBadInvOrder.h"
#include "CCapeBoundaries.h"
#include "CCapeComputation.h"
#include "CCapeData.h"
#include "CCapeFailedInitialisation.h"
#include "CCapeIllegalAccess.h"
#include "CCapeImplementation.h"
#include "CCapeInvalidArgument.h"
#include "CCapeInvalidOperation.h"
#include "CCapeLicenceError.h"
#include "CCapeLimitedImpl.h"
#include "CCapeNoImpl.h"
#include "CCapeNoMemory.h"
#include "CCapeOutOfBounds.h"
#include "CCapeOutOfResources.h"
#include "CCapePersistence.h"
#include "CCapePersistenceNotFound.h"
#include "CCapePersistenceOverflow.h"
#include "CCapePersistenceSystemError.h"
#include "CCapeSolvingError.h"
#include "CCapeTimeOut.h"
#include "CCapeUnknown.h"
#include "CCapeUser.h"




namespace R2PException{
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CMyException::CMyException()
{
	_refError = new srefUnion;
	_refError->_ref = 1;
	_refError->_COCode = 0;
	_refError->COEx = NULL;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CMyException::CMyException(HRESULT error_code, const std::wstring& name, const wchar_t* desc, ...)
{
	_refError = new srefUnion;
	_refError->_ref = 1;
	_refError->COEx = NULL;
	_refError->_COCode = Utils::HRES2CapeError(error_code, &(_refError->COEx));
	_refError->COEx->set_name(name);
	std::wstring partialDesc;
	_refError->COEx->get_description(partialDesc);
	//Rebuild the va_list in order to pass it to wsprintf
	va_list marker;
	va_start( marker, desc );

	size_t nbElem = _vscwprintf( desc, marker ) + 1;

	wchar_t* buffer = (wchar_t*)alloca(sizeof(wchar_t) * nbElem);
	memset(buffer, 0, sizeof(wchar_t) * nbElem);
	vswprintf_s( buffer, /*_countof( buffer )*/nbElem, desc, marker );

	va_end( marker );
	
	partialDesc += buffer;
	_refError->COEx->set_description(partialDesc);
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CMyException::CMyException(HRESULT error_code, const IDispatchPtr& pdis){
	_refError = new srefUnion;
	_refError->_ref = 1;
	_refError->COEx = Utils::OutScope2InScope(error_code, pdis);
	_refError->_COCode = error_code;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CMyException::CMyException(const CMyException& _ex){
	_refError = _ex._refError;
	++_refError->_ref;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CMyException::~CMyException()
{
	if(--_refError->_ref == 0){
		if (_refError->COEx) delete _refError->COEx;
		delete _refError;
		_refError = NULL;
	}
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
HRESULT CMyException::get_COCode(long& code){
	code = _refError->_COCode;

	return S_OK;
}
/****************************************************************************************************************************
*								from ECapeRoot
*
****************************************************************************************************************************/
HRESULT CMyException::get_name(std::wstring& name){
	HRESULT hres = S_OK;
	if(_refError->COEx)
		_refError->COEx->get_name(name);
	else{
		name.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeRoot
*
****************************************************************************************************************************/
HRESULT CMyException::set_name(const std::wstring& name){
	HRESULT hres = S_OK;
	
	_refError->COEx != NULL ? _refError->COEx->set_name(name) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::get_legacyCode(long& code){
	HRESULT hres = S_OK;

	_refError->COEx != NULL ? _refError->COEx->get_code(code) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::set_description(const std::wstring& description){
	HRESULT hres = S_OK;
	_refError->COEx != NULL ? _refError->COEx->set_description(description) : hres = E_FAIL;
	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::get_description(std::wstring& description){
	HRESULT hres = S_OK;

	if(_refError->COEx)
		_refError->COEx->get_description(description);
	else{
		description.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
HRESULT CMyException::add_description(const wchar_t* desc, ...){
	HRESULT hres = S_OK;

	if(_refError->COEx){
		//Rebuild the va_list in order to pass it to wsprintf
		va_list marker;
		va_start( marker, desc );

		wchar_t buffer[1024] = { 0 };
		vswprintf_s( buffer, _countof( buffer ), desc, marker );

		va_end( marker );

		_refError->COEx->add_description(buffer);
	}
	else
		hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::get_scope(std::wstring& scope){
	HRESULT hres = S_OK;
	
	if(_refError->COEx)
		_refError->COEx->get_scope(scope);
	else{
		scope.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::set_scope(const std::wstring& scope){
	HRESULT hres = S_OK;
	
	_refError->COEx ? _refError->COEx->set_scope(scope) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::get_interfaceName(std::wstring& interfaceName){
	HRESULT hres = S_OK;

	if(_refError->COEx)
		_refError->COEx->get_interfaceName(interfaceName);
	else{
		interfaceName.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::set_interfaceName(const std::wstring& interfaceName){
	HRESULT hres = S_OK;

	_refError->COEx ? _refError->COEx->set_interfaceName(interfaceName) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::get_operation(std::wstring& operation){
	HRESULT hres = S_OK;

	if(_refError->COEx)
		_refError->COEx->get_operation(operation);
	else{
		operation.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeUser
*
****************************************************************************************************************************/
HRESULT CMyException::set_operation(const std::wstring& operation){
	HRESULT hres = S_OK;

	_refError->COEx ? _refError->COEx->set_operation(operation) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*						from ECapeUser		
*
****************************************************************************************************************************/
HRESULT CMyException::get_moreInfo(std::wstring& moreInfo){
	HRESULT hres = S_OK;

	if(_refError->COEx)
		_refError->COEx->get_moreInfo(moreInfo);
	else{
		moreInfo.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*						from ECapeUser		
*
****************************************************************************************************************************/
HRESULT CMyException::set_moreInfo(const std::wstring& moreInfo){
	HRESULT hres = S_OK;

	_refError->COEx ? _refError->COEx->set_moreInfo(moreInfo) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*				from ECapeBadCOParameter				
*
****************************************************************************************************************************/
HRESULT CMyException::get_parameterName(std::wstring& ParamName){
	HRESULT hres = S_OK;

	CCapeBadCOParameter* pError = dynamic_cast<CCapeBadCOParameter*>(_refError->COEx);
	if(pError)
		pError->get_parameterName(ParamName);
	else{
		ParamName.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*				from ECapeBadCOParameter				
*
****************************************************************************************************************************/
HRESULT CMyException::set_parameterName(const std::wstring& ParamName){
	HRESULT hres = S_OK;

	CCapeBadCOParameter* pError = dynamic_cast<CCapeBadCOParameter*>(_refError->COEx);
	pError != NULL ? pError->set_parameterName(ParamName) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*							from ECapeBadCOParameter	
*
****************************************************************************************************************************/
HRESULT CMyException::get_parameter(IDispatch** theParam){
	HRESULT hres = S_OK;

	CCapeBadCOParameter* pError = dynamic_cast<CCapeBadCOParameter*>(_refError->COEx);
	if(pError){
		//No need to addref: the get_parameter method do it for us
		pError->get_parameter(theParam);
	}
	else{
		*theParam = NULL;
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*							from ECapeBadCOParameter	
*
****************************************************************************************************************************/
HRESULT CMyException::set_parameter(const IDispatch* theParam){
	HRESULT hres = S_OK;

	CCapeBadCOParameter* pError = dynamic_cast<CCapeBadCOParameter*>(_refError->COEx);
	//No need to addref: the set_parameter method do it for us
	pError != NULL ? pError->set_parameter(theParam) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBadArgument
*
****************************************************************************************************************************/
HRESULT CMyException::get_position(short& pos){
	HRESULT hres = S_OK;

	CCapeBadArgument* pError = dynamic_cast<CCapeBadArgument*>(_refError->COEx);
	if(pError){
		pError->get_position(pos);
	}
	else{
		pos = -1;
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBadArgument
*
****************************************************************************************************************************/
HRESULT CMyException::set_position(const short& pos){
	HRESULT hres = S_OK;

	CCapeBadArgument* pError = dynamic_cast<CCapeBadArgument*>(_refError->COEx);
	pError ? pError->set_position(pos) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBoundaries
*
****************************************************************************************************************************/
HRESULT CMyException::get_LowerBound(double& lbound){
	HRESULT hres = S_OK;

	CCapeBoundaries* pError = dynamic_cast<CCapeBoundaries*>(_refError->COEx);
	if(pError){
		pError->get_LowerBound(lbound);
	}
	else{
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBoundaries
*
****************************************************************************************************************************/
HRESULT CMyException::get_UpperBound(double& ubound){
	HRESULT hres = S_OK;

	CCapeBoundaries* pError = dynamic_cast<CCapeBoundaries*>(_refError->COEx);
	if(pError){
		pError->get_UpperBound(ubound);
	}
	else{
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBoundaries
*
****************************************************************************************************************************/
HRESULT CMyException::get_value(double& value){
	HRESULT hres = S_OK;

	CCapeBoundaries* pError = dynamic_cast<CCapeBoundaries*>(_refError->COEx);
	if(pError){
		pError->get_value(value);
	}
	else{
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBoundaries
*
****************************************************************************************************************************/
HRESULT CMyException::get_Type(std::wstring& type){
	HRESULT hres = S_OK;

	CCapeBoundaries* pError = dynamic_cast<CCapeBoundaries*>(_refError->COEx);
	if(pError){
		pError->get_Type(type);
	}
	else{
		type.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBoundaries
*
****************************************************************************************************************************/
HRESULT CMyException::set_Bound(const double& lbound, const double& ubound, const double& value, const std::wstring& type){
	HRESULT hres = S_OK;

	CCapeBoundaries* pError = dynamic_cast<CCapeBoundaries*>(_refError->COEx);
	pError ? pError->set_boundaries(lbound, ubound, value, type) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBadInvOrder
*
****************************************************************************************************************************/
HRESULT CMyException::get_requestedOperation(std::wstring& ReqOp){
	HRESULT hres = S_OK;

	CCapeBadInvOrder* pError = dynamic_cast<CCapeBadInvOrder*>(_refError->COEx);
	if(pError){
		pError->get_requestedOperation(ReqOp);
	}
	else{
		ReqOp.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapeBadInvOrder
*
****************************************************************************************************************************/
HRESULT CMyException::set_requestedOperation(const std::wstring& ReqOp){
	HRESULT hres = S_OK;

	CCapeBadInvOrder* pError = dynamic_cast<CCapeBadInvOrder*>(_refError->COEx);
	pError ? pError->set_requestedOperation(ReqOp) : hres = E_FAIL;

	return hres;
}
/****************************************************************************************************************************
*								from ECapePersistenceNotFound
*
****************************************************************************************************************************/
HRESULT CMyException::get_itemName(std::wstring& iName){
	HRESULT hres = S_OK;

	CCapePersistenceNotFound* pError = dynamic_cast<CCapePersistenceNotFound*>(_refError->COEx);
	if(pError){
		pError->get_itemName(iName);
	}
	else{
		iName.clear();
		hres = E_FAIL;
	}

	return hres;
}
/****************************************************************************************************************************
*								from ECapePersistenceNotFound
*
****************************************************************************************************************************/
HRESULT CMyException::set_itemName(const std::wstring& iName){
	HRESULT hres = S_OK;

	CCapePersistenceNotFound* pError = dynamic_cast<CCapePersistenceNotFound*>(_refError->COEx);
	pError ? pError->set_itemName(iName) : hres = E_FAIL;

	return hres;
}



}
