#if !defined(__CCOMUTILS_H__)
#define __CCOMUTILS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"
#include <comdef.h>
//#include <constants.h>
#include <string>

/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/

/// <summary>some useful method relating to COM</summary>
class CCOMUtils{
public:
	/// <summary>check pointer and release it if not null: this method is only use to decrease Cyclomatic Complexity</summary>
	/// <param name="pDisp">Interface to release</param>
	/// <returns>Count reference</returns>
	static /*inline */int checkNRelease(IDispatch* pDisp) throw();

	/// <summary>Initialize Variant to receive a copy from another one using CopyVariant (which is very restricting)</summary>
	/// <param name="fromVariant">variant to copy: check if type is corresponding to vt</param>
	/// <param name="toVariant">the variant that will receive the copy</param>
	/// <param name="vt">type of variant</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>HRESULT, S_OK for success R2PException::R2P_SWARN on warning</returns>
	static /*inline */HRESULT InitVariant4Copy(const VARIANT& fromVariant, VARIANT& toVariant, VARTYPE vt) throw(...);

	/// <summary>Load the COM library depending on registry value</summary>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	static /*inline */void loadCOM() throw(...);

	/// <summary>Convert DOUBLE_INF to 0., otherwise return val</summary>
	/// <returns>Converted value</returns>
	//static inline void checkUndefinedValue(const double* val, double*& val_out) throw();

	/// <summary>Counterpart method of checkUndefinedValue: return value passed</summary>
	/// <returns>input value</returns>
	//template<class T> static inline void nop(const T* val, T*& val_out) throw();
};

#if 0
/****************************************************************************************************************************
*	inline method
*
****************************************************************************************************************************/
void CCOMUtils::checkUndefinedValue(const double* val, double*& val_out){
	*val_out = isNaN(*val)? 0.: *val;
}
/****************************************************************************************************************************
*	inline method
*
****************************************************************************************************************************/
template <class T> void CCOMUtils::nop(const T* val, T*& val_out){
	//return val;
	val_out = const_cast<T*>(val);
}
#endif
#endif  //  __CCOMUTILS_H__
