// CCapeBadArgument.h: interface for the ECapeBadArgument class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEBADARGUMENT_H__)
#define __CCAPEBADARGUMENT_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CCapeData.h"
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: An argument value of the operation is not correct.</summary> 
	class CCapeBadArgument: public CCapeData
	{
	public:
		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapeBadArgument(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapeBadArgument object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeBadArgument interface</param>
		CCapeBadArgument(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		virtual ~CCapeBadArgument();

		/// <summary>get the position from collection where argument is bad</summary>
		/// <param name="pos">the position</param>
		/// <returns>void</returns>
		void get_position(short& pos);

		/// <summary>set the position from collection where argument is bad</summary>
		/// <param name="pos">the position</param>
		/// <returns>void</returns>
		void set_position(short pos);

	protected:
		/// <summary>The position of the argument value within the signature of the operation</summary>
		/// <remarks>First argument is as position 1</remarks>
		short _position;
	};
}
#endif // __CCAPEBADARGUMENT_H__
