// CCapeBadInvOrder.h: interface for the ECapeBadInvOrder class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPEBADINVORDER_H__)
#define __CCAPEBADINVORDER_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CCapeComputation.h"
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: The necessary pre-requisite operation has not been called prior to the operation request.</summary> 
	class CCapeBadInvOrder: public CCapeComputation
	{
	public:
		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapeBadInvOrder(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapeBadInvOrder object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeBadInvOrder interface</param>
		CCapeBadInvOrder(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		~CCapeBadInvOrder();

		/// <summary>set error specific class attribute</summary>
		/// <param name="ReqOp">Name of the prerequisite operation</param>
		/// <returns>void</returns>
		void set_requestedOperation(const std::wstring& ReqOp);

		/// <summary>set error specific class attribute</summary>
		/// <param name="ReqOp">Name of the prerequisite operation</param>
		/// <returns>void</returns>
		void get_requestedOperation(std::wstring& ReqOp);

	protected:
		/// <summary>The necessary prerequisite operation</summary>
		std::wstring _requestedOperation;

	};
}
#endif // __CCAPEBADINVORDER_H__
