// CCapeLimitedImpl.h: interface for the ECapeLimitedImpl class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(__CCAPELIMITEDIMPL_H__)
#define __CCAPELIMITEDIMPL_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "CCapeImplementation.h"
#include "stdafx.h"

namespace R2PException{
	/// <summary>Exception handling class: The limit of the implementation has been violated.
	/// <para>An operation may be partially implemented for example a Property Package could implement TP flash but not PH flash. If a caller requests for a PH flash, then this error indicates that some flash calculations are supported but not the requested one.</para>
	/// <para>The factory can only create one instance (because the component is an evaluation copy), when the caller requests for a second creation this error shows that this implementation is limited.</para>
	/// </summary> 
	class CCapeLimitedImpl: public CCapeImplementation
	{
	public:
		/// <summary>Constructor</summary>
		/// <param name="hr">legacy error code</param>
		/// <param name="desc">default error description</param>
		CCapeLimitedImpl(HRESULT hr, const std::wstring& desc = L"");

		/// <summary>Constructor: create a CCapeLimitedImpl object from a CAPE-OPEN component</summary>
		/// <param name="pDisp">pointer to a CAPE-OPEN component which have to expose ECapeLimitedImpl interface</param>
		CCapeLimitedImpl(IDispatch* pDisp);

		/// <summary>Destructor</summary>
		~CCapeLimitedImpl();

	};
}
#endif // __CCAPELIMITEDIMPL_H__
