// HydroServer_IFP.cpp: implmentation de CHydroServer_IFP

#include "stdafx.h"
#include "HydroServer_IFP.h"

#include "CDATAPACK.h"
#include "CCapeError.h"

// CHydroServer_IFP

STDMETHODIMP CHydroServer_IFP::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IDispatch
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CHydroServer_IFP::CHydroServer_IFP() :
	m_superficialVelocities(3,0.), m_densities(3,0.), m_viscosities(3,0.), m_heatCapacities(3,0.),
	m_conductivities(3,0.), m_surfaceTensions(3,3,0.),
	m_isReady(false)
{
	m_strName = L"Hydro Server";
	m_strDescription = L"Server for HPM Simple Hydro calculations";
}

/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
CHydroServer_IFP::~CHydroServer_IFP(){
	m_strName.clear();
	m_strDescription.clear();
	::MessageBox(NULL,L"Hope to see you soon",L"Bye Bye", MB_ICONINFORMATION);
}
/****************************************************************************************************************************
*								ICapeIdentification
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::get_ComponentName(BSTR* name){
	CDATAPACK::wstring2BSTR(m_strName, *name);
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::put_ComponentName(BSTR name){
	CDATAPACK::BSTR2WString(name, m_strName);
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::get_ComponentDescription(BSTR* description){
	CDATAPACK::wstring2BSTR(m_strDescription, *description);
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::put_ComponentDescription(BSTR description){
	m_strDescription = description;
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
//STDMETHODIMP CHydroServer_IFP::SetHPM(IDispatch* hpminterface){
//	m_HPMinterface = hpminterface;
//	return S_OK;
//}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
//STDMETHODIMP CHydroServer_IFP::GetHPMPackageList(VARIANT* hpmPackages){
//	HRESULT hr = S_OK;
//	try{
//		;
//	} catch(R2PException::CMyException& herr){
//		hr = generateException(herr, L"ICapeHydroPackageManager", L"GetHPMPackageList");
//	}
//	return hr;
//
//}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
//STDMETHODIMP CHydroServer_IFP::GetHPMPackage(BSTR HPMName, IDispatch **HPMPackage){
//	HRESULT hr = S_OK;
//	try{
//		;
//	} catch(R2PException::CMyException& herr){
//		hr = generateException(herr, L"ICapeHydroPackageManager", L"GetHPMPackage");
//	}
//	return hr;
//}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetNumPhases(LONG* num){
	*num = 3;
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetPhaseInfo(LONG numPhase, BSTR* stateOfAggregation, BSTR* typeInfo){
	HRESULT hr = S_OK;
	switch (numPhase) {
		case 1:
			CDATAPACK::wstring2BSTR(L"Vapor", *stateOfAggregation);
			*typeInfo = NULL;
			break;
		case 2:
			CDATAPACK::wstring2BSTR(L"Liquid", *stateOfAggregation);
			CDATAPACK::wstring2BSTR(L"Organic", *typeInfo);
			break;
		case 3:
			CDATAPACK::wstring2BSTR(L"Liquid", *stateOfAggregation);
			CDATAPACK::wstring2BSTR(L"Aqueous", *typeInfo);
			break;
		default:
			*stateOfAggregation = NULL;
			*typeInfo = NULL;
			hr = generateException(R2PException::CMyException(ECapeOutOfBoundsHR,this), L"ICapeHydroPhases", L"GetPhaseInfo");
			break;
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CalcFlow(BSTR specification1,BSTR specification2,BSTR flowregime){
	HRESULT hr = S_OK;
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	if (spec1 == L"Us") {
		TNT::Array1D<EPhases> phaseQualifiers(m_nbrOfPresentPhases);
		std::map<std::wstring,EPhases>::const_iterator phaseLabel;
		int i = 0;
		for (phaseLabel=m_phaseLabels.begin(); phaseLabel!=m_phaseLabels.end(); phaseLabel++, i++) {
			phaseQualifiers[i] = (*phaseLabel).second;
		}

		m_hydro.hydroUk(
			m_diameter, m_roughness, m_inclination, 
			m_nbrOfPresentPhases, phaseQualifiers, 
			m_superficialVelocities, m_densities, m_viscosities, m_surfaceTensions, m_heatCapacities, m_conductivities);

		m_hydro.computeProperties();

		m_isReady = true;

	} else {
		hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFlowRoutine", L"CalcFlow");
	}

	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CheckCalcFlowSpec(BSTR specification1,BSTR specification2,BSTR flowregime, VARIANT_BOOL* isSupported){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	if (spec1 == L"Us") {
		*isSupported = VARIANT_TRUE;
		if (flowregime != NULL)
			*isSupported = VARIANT_FALSE;
	} else {
		*isSupported = VARIANT_FALSE;
	}

	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetFinalState(VARIANT* propvals){
	HRESULT hr = S_OK;
	try {
		CDATAPACK::Array2Variant(m_anonymous, *propvals);
	} catch (...) {
		hr = generateException(R2PException::CMyException(ECapeUnknownHR,this), L"ICapeHydroFlowRoutine", L"GetFinalState");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetInitState(VARIANT propvals){
	HRESULT hr = S_OK;
	try {
		CDATAPACK::Variant2Array(propvals, m_anonymous);
	} catch (...) {
		hr = generateException(R2PException::CMyException(ECapeUnknownHR,this), L"ICapeHydroFlowRoutine", L"SetInitState");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CalcSingleHydroProp(VARIANT props,BSTR phaseLabel){
	TNT::Array1D<std::wstring> pty;
	std::wstring label;
	CDATAPACK::BSTR2WString(phaseLabel, label);
	CDATAPACK::Variant2Array(props, pty);

	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CalcTwoHydroProp(VARIANT props,VARIANT phaseLabels){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CheckMixtureHydroPropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		TNT::Array1D<std::wstring> pty(3);
		pty[0] = L"pressureDropFrictional";
		pty[1] = L"pressureDropGravity";
		pty[2] = L"pressureDropTotal";
		CDATAPACK::Array2Variant(pty, *props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"CheckMixtureHydroPropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CheckSingleHydroPropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		TNT::Array1D<std::wstring> pty(3);
		pty[0] = L"absoluteVelocity";
		pty[1] = L"superficialVelocity";
		pty[2] = L"volumeFraction";
		CDATAPACK::Array2Variant(pty, *props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"CheckSingleHydroPropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CheckTwoHydroPropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		VariantInit(props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"CheckTwoHydroPropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetGeometryPropList(BSTR specification1,BSTR specification2,VARIANT *props){
	TNT::Array1D<std::wstring> pty(3);
	pty[0] = L"diameter";
	pty[1] = L"inclination";
	pty[2] = L"roughness";
	CDATAPACK::Array2Variant(pty, *props);
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSingleHydroPropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		TNT::Array1D<std::wstring> pty(1);
		pty[0] = L"superficialVelocity";
		CDATAPACK::Array2Variant(pty, *props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"GetSingleHydroPropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSinglePhasePropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		TNT::Array1D<std::wstring> pty(1);
		pty[0] = L"density";
		CDATAPACK::Array2Variant(pty, *props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"GetSinglePhasePropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetTwoPhasePropList(BSTR specification1,BSTR specification2,VARIANT *props){
	std::wstring spec1, spec2;
	CDATAPACK::BSTR2WString(specification1, spec1);
	CDATAPACK::BSTR2WString(specification2, spec2);

	HRESULT hr = S_OK;
	if (spec1 == L"Us") {
		VariantInit(props);
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroPropertyRoutine", L"GetTwoPhasePropList");
	}
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetFieldInfo( BSTR layerLabel, BSTR fieldLabel, BSTR fieldAttribute, BSTR* value){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetFieldInfo");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetLayerInfo( BSTR layerLabel, BSTR fieldAttribute, BSTR* value){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetLayerInfo");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetPresentFields( BSTR layerLabel,VARIANT* fieldLabels){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetPresentFields");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetPresentLayers(VARIANT* layerLabels){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetPresentLayers");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetPresentPhases( BSTR layerLabel, BSTR fieldLabel,VARIANT* phaseLabels){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetPresentPhases");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSingleFieldProp( BSTR property, BSTR layerLabel, BSTR fieldLabel,double* result){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetSingleFieldProp");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSingleLayerProp( BSTR property, BSTR layerLabel,double* result){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetSingleLayerProp");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSinglePhaseProp( BSTR property, BSTR layerLabel, BSTR fieldLabel, BSTR phaseLabel,double* result){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetSinglePhaseProp");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetTwoLayerProp( BSTR property, VARIANT layerLabels,double* result){
	HRESULT hr = generateException(R2PException::CMyException(ECapeNoImplHR,this), L"ICapeHydroFields", L"GetTwoLayerProp");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::ClearAllProps(){
	m_isReady = false;
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CreateHPM(IDispatch**hpm){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::CopyFromHPM(IDispatch*hpmSource){
	m_HPMinterface = hpmSource;
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetFlowRegime(BSTR*regime,BSTR* qualifier){
	EHydroFlowPatterns pattern = m_hydro.getFlowPattern();
	switch (pattern) {
		case MONO_GAS:
		case MONO_OIL:
		case MONO_WAT:
			CDATAPACK::wstring2BSTR(L"Homogeneous", *regime);
			CDATAPACK::wstring2BSTR(L"SinglePhase", *qualifier);
			break;

		case LIQ_LIQ:
			CDATAPACK::wstring2BSTR(L"Homogeneous", *regime);
			CDATAPACK::wstring2BSTR(L"TwoPhase", *qualifier);
			break;

		case STRA_SMO:
		case ST_SLUG:
			CDATAPACK::wstring2BSTR(L"Separated", *regime);
			CDATAPACK::wstring2BSTR(L"StratifiedSmooth", *qualifier);
			break;

		case STRA_WAV:
			CDATAPACK::wstring2BSTR(L"Separated", *regime);
			CDATAPACK::wstring2BSTR(L"StratifiedWavy", *qualifier);
			break;

		case ANNULAR:
			CDATAPACK::wstring2BSTR(L"Separated", *regime);
			CDATAPACK::wstring2BSTR(L"Annular", *qualifier);
			break;

		case DISPERSE:
			CDATAPACK::wstring2BSTR(L"Bubble", *regime);
			*qualifier = NULL;
			break;

		case SLUGFLOW:
			CDATAPACK::wstring2BSTR(L"Slug", *regime);
			*qualifier = NULL;
			break;

		default:
			*regime = NULL;
			*qualifier = NULL;
			break;
	}

	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetMixtureProp(BSTR pty,VARIANT* values){
	std::wstring prop;
	CDATAPACK::BSTR2WString(pty, prop);

	HRESULT hr = S_OK;
	VariantInit(values);

	TNT::Array1D<double> result(1, 0.);
	if (!m_isReady) {
		hr = generateException(R2PException::CMyException(ECapeUnknownHR,this), L"ICapeHydroFlow", L"GetMixtureProp");

	} else if (prop == L"pressureDropFrictional") {
		result[0] = m_hydro.getFrictionPressureDrop();
		CDATAPACK::Array2Variant(result, *values);

	} else if (prop == L"pressureDropGravity") {
		result[0] = m_hydro.getGravityPressureDrop();
		CDATAPACK::Array2Variant(result, *values);

	} else if (prop == L"pressureDropTotal") {
		result[0] = m_hydro.getLocalPressureDrop();
		CDATAPACK::Array2Variant(result, *values);

	} else {
		hr = generateException(R2PException::CMyException(ECapeBadInvOrderHR,this), L"ICapeHydroFlow", L"GetMixtureProp");
	}

	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSingleHydroProp(BSTR pty, BSTR phaseLabel, VARIANT* values){
	std::wstring prop, label;
	CDATAPACK::BSTR2WString(pty, prop);
	CDATAPACK::BSTR2WString(phaseLabel, label);

	HRESULT hr = S_OK;
	VariantInit(values);

	TNT::Array1D<double> result(1, 0.);
	if (!m_isReady) {
		hr = generateException(R2PException::CMyException(ECapeUnknownHR,this), L"ICapeHydroFlow", L"GetSingleHydroProp");

	} else if (prop == L"absoluteVelocity") {
		const TNT::Array1D<double>* absVelo = m_hydro.getAbsoluteVelocities();
		result[0] = (*absVelo)[m_phaseLabels[label]];
		CDATAPACK::Array2Variant(result, *values);

	} else if (prop == L"superficialVelocity") {
		const TNT::Array1D<double>* supVelo = m_hydro.getSuperficialVelocities();
		result[0] = (*supVelo)[m_phaseLabels[label]];
		CDATAPACK::Array2Variant(result, *values);

	} else if (prop == L"volumeFraction") {
		const TNT::Array1D<double>* volFrac = m_hydro.getLocalVolumetricFractions();
		result[0] = (*volFrac)[m_phaseLabels[label]];
		CDATAPACK::Array2Variant(result, *values);

	} else {
		hr = generateException(R2PException::CMyException(ECapeBadInvOrderHR,this), L"ICapeHydroFlow", L"GetSingleHydroProp");
	}


	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetTwoHydroProp(BSTR pty,VARIANT phaseLabel,VARIANT* values){
	HRESULT hr = S_OK;
	if (!m_isReady)
		hr = generateException(R2PException::CMyException(ECapeUnknownHR,this), L"ICapeHydroFlow", L"GetTwoHydroProp");
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetGeometryProp(BSTR pty,VARIANT values){
	std::wstring spec;
	TNT::Array1D<double> value(1);
	CDATAPACK::BSTR2WString(pty, spec);
	CDATAPACK::Variant2Array(values, value);

	HRESULT hr = S_OK;
	if (spec == L"diameter") {
		m_diameter = value[0];
	} else if (spec == L"inclination") {
		m_inclination = value[0];
	} else if (spec == L"roughness") {
		m_roughness = value[0];
	} else {
		hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroFlow", L"SetGeometryProp");
	}

	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetMixtureProp(BSTR pty,VARIANT values){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetOverallProp(BSTR pty,VARIANT *values){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetPresentPhases(VARIANT phaseLabel,VARIANT stateOfAggregation,VARIANT typeInfo){
	TNT::Array1D<std::wstring> labels;
	TNT::Array1D<std::wstring> states;
	TNT::Array1D<std::wstring> infos;
	CDATAPACK::Variant2Array(phaseLabel,         labels);
	CDATAPACK::Variant2Array(stateOfAggregation, states);
	CDATAPACK::Variant2Array(typeInfo,           infos);

	m_phaseLabels.clear();
	m_nbrOfPresentPhases = labels.dim1();

	HRESULT hr = S_OK;
	for (int i=0; i<m_nbrOfPresentPhases; i++) {
		std::wstring& label = labels[i];
		std::wstring& state = states[i];
		std::wstring& info  = infos[i];

		if (state == L"Vapor") {
			m_phaseLabels[label] = GAS;
		} else if (state == L"Liquid") {
			if (info == L"Organic") {
				m_phaseLabels[label] = OIL;
			} else if (info == L"Aqueous") {
				m_phaseLabels[label] = WATER;
			} else /* Hydrate ou UNDEFINED */ {
				m_phaseLabels[label] = OIL;
			}
		} else if (state == L"Solid") {
			m_phaseLabels[label] = OIL;
		} else if (state == L"Unknown") {
			m_phaseLabels[label] = OIL;
		} else {
			hr = generateException(R2PException::CMyException(ECapeInvalidArgumentHR,this), L"ICapeHydroFlow", L"SetPresentPhases");
		}
	}

	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetSingleHydroProp(BSTR pty,BSTR phaseLabel,VARIANT values){
	std::wstring prop;
	std::wstring label;
	TNT::Array1D<double> value;
	CDATAPACK::BSTR2WString(pty, prop);
	CDATAPACK::BSTR2WString(phaseLabel, label);
	CDATAPACK::Variant2Array(values, value);

	if (prop == L"superficialVelocity") {
		m_superficialVelocities[m_phaseLabels[label]] = value[0];
	}

	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetSinglePhaseProp(BSTR pty,BSTR phaseLabel,VARIANT values){
	std::wstring prop;
	std::wstring label;
	TNT::Array1D<double> value;
	CDATAPACK::BSTR2WString(pty, prop);
	CDATAPACK::BSTR2WString(phaseLabel, label);
	CDATAPACK::Variant2Array(values, value);

	if (prop == L"density") {
		m_densities[m_phaseLabels[label]] = value[0];
	} else if (prop == L"heatCapacityCp") {
		m_heatCapacities[m_phaseLabels[label]] = value[0];
	} else if (prop == L"ThermalConductivity") {
		m_conductivities [m_phaseLabels[label]] = value[0];
	} else if (prop == L"viscosity") {
		m_viscosities[m_phaseLabels[label]] = value[0];
	}

	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetTwoHydroProp(BSTR pty,VARIANT phaseLabel,VARIANT values){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::SetTwoPhaseProp(BSTR pty,VARIANT phaseLabel,VARIANT values){
	return S_OK;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
HRESULT CHydroServer_IFP::generateException(R2PException::CMyException& herr, const std::wstring& interfaceName, const std::wstring operation){
	HRESULT hr;
	//No need to check if herr is equal to mCOError: mCOError implements a garbage collector.
	if(mCOError) delete mCOError;
	mCOError = new R2PException::CMyException(herr);
	mCOError->set_operation(operation);
	mCOError->set_interfaceName(interfaceName);
	mCOError->get_COCode(hr);
	return hr;
}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::Load(IStream* pstm){
	HRESULT hr = S_OK;
	try{
		;
	}catch(R2PException::CMyException& herr){
		hr = generateException(herr, L"Persistence interface IStream", L"Load");
	}
	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::Save(IStream* pstm, BOOL fClearDirty){
	HRESULT hr = S_OK;
	try{
		;
	}catch(R2PException::CMyException& herr){
		hr = generateException(herr, L"Persistence interface IStream", L"Save");
	}
	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetSizeMax(_ULARGE_INTEGER* pcbSize){
	return E_NOTIMPL;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::IsDirty(){
	HRESULT hr = S_OK;
	try{
		;
	}catch(R2PException::CMyException& herr){
		hr = generateException(herr, L"Persistence interface IStream", L"IsDirty");
	}
	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
STDMETHODIMP CHydroServer_IFP::GetClassID(GUID *pClassID){
	HRESULT hr = S_OK;
	try{
		;
	}catch(R2PException::CMyException& herr){
		hr = generateException(herr, L"Persistence interface IStream", L"IsDirty");
	}
	return hr;
}
/****************************************************************************************************************************
*
*
****************************************************************************************************************************/
